@@ -29,8 +29,12 @@
 #include <glib.h>
 #include <glib/gi18n.h>
 #include <gobject/gvaluecollector.h>
+#include <gtk/gtkiconfactory.h>
+#include <gtk/gtkicontheme.h>
+#include <gtk/gtkstock.h>
 
 #include "gossip-account.h"
+#include "gossip-stock.h"
 #include "gossip-utils.h"
 
 #include "libgossip-marshal.h"
@@ -46,19 +50,14 @@ struct _GossipAccountPriv {
 	gchar             *name;
 	gboolean           auto_connect;
 	gboolean           use_proxy;
-#ifdef HAVE_TELEPATHY
-	gchar             *protocol;
-	gchar             *cmgr_name;
-#endif
 
 	GHashTable        *parameters;
 };
 
 typedef struct {
-	GossipAccountParamFunc  callback;
-	gpointer                user_data;
-	GossipAccount          *account;
-} GossipAccountParamData;
+	GossipAccountType  account_type;
+	GList             *params;
+} GetParamsData;
 
 static void           account_class_init            (GossipAccountClass     *class);
 static void           account_init                  (GossipAccount          *account);
@@ -82,14 +81,12 @@ static void           account_param_get_valist      (GossipAccount          *acc
 						     va_list                 var_args);
 static void           account_param_get_all_foreach (gchar                  *param_name,
 						     GossipAccountParam     *param,
-						     GList                 **params);
-static void           account_param_foreach_foreach (const gchar            *param_name,
-						     GossipAccountParam     *param,
-						     GossipAccountParamData *data);
+						     GetParamsData          *data);
 static void           account_param_free            (GossipAccountParam     *param);
+const gchar *         account_param_name_convert    (GossipAccountType       type,
+						     const gchar            *param_name);
 static void           account_set_type              (GossipAccount          *account,
 						     GossipAccountType       type);
-const gchar *         account_param_name_convert    (const gchar            *param_name);
 
 enum {
 	PROP_0,
@@ -97,10 +94,6 @@ enum {
 	PROP_NAME,
 	PROP_AUTO_CONNECT,
 	PROP_USE_PROXY,
-#ifdef HAVE_TELEPATHY
-	PROP_PROTOCOL,
-	PROP_CMGR_NAME,
-#endif
 };
 
 static gpointer  parent_class = NULL;
@@ -148,9 +141,9 @@ account_class_init (GossipAccountClass *class)
 					 g_param_spec_int ("type",
 							   "Account Type",
 							   "The account protocol type, e.g. MSN",
-							   G_MININT,
-							   G_MAXINT,
-							   GOSSIP_ACCOUNT_TYPE_JABBER,
+							   GOSSIP_ACCOUNT_TYPE_JABBER_LEGACY,
+							   GOSSIP_ACCOUNT_TYPE_UNKNOWN,
+							   GOSSIP_ACCOUNT_TYPE_JABBER_LEGACY,
 							   G_PARAM_READWRITE));
 
 	g_object_class_install_property (object_class,
@@ -176,23 +169,6 @@ account_class_init (GossipAccountClass *class)
 							       "Identifies if the connection uses the environment proxy",
 							       FALSE,
 							       G_PARAM_READWRITE));
-#ifdef HAVE_TELEPATHY
-	g_object_class_install_property (object_class,
-					 PROP_PROTOCOL,
-					 g_param_spec_string ("protocol",
-							      "Protocol",
-							      "Protocol this account uses",
-							      NULL,
-							      G_PARAM_READWRITE));
-
-	g_object_class_install_property (object_class,
-					 PROP_CMGR_NAME,
-					 g_param_spec_string ("cmgr_name",
-							      "Connection Manager",
-							      "The connection manager that provides this account",
-							      NULL,
-							      G_PARAM_READWRITE));
-#endif
 
 	g_type_class_add_private (object_class, sizeof (GossipAccountPriv));
 }
@@ -208,15 +184,11 @@ account_init (GossipAccount *account)
 	priv->name         = NULL;
 	priv->auto_connect = TRUE;
 	priv->use_proxy    = FALSE;
-#ifdef HAVE_TELEPATHY
-	priv->protocol     = NULL;
-	priv->cmgr_name    = NULL;
-#endif
 
 	priv->parameters = g_hash_table_new_full (g_str_hash,
 						  g_str_equal,
 						  g_free,
-						  (GDestroyNotify)account_param_free);
+						  (GDestroyNotify) account_param_free);
 }
 
 static void
@@ -227,10 +199,7 @@ account_finalize (GObject *object)
 	priv = GET_PRIV (object);
 
 	g_free (priv->name);
-#ifdef HAVE_TELEPATHY
-	g_free (priv->protocol);
-	g_free (priv->cmgr_name);
-#endif
+
 	g_hash_table_unref (priv->parameters);
 
 	(G_OBJECT_CLASS (parent_class)->finalize) (object);
@@ -261,14 +230,6 @@ account_get_property (GObject    *object,
 	case PROP_USE_PROXY:
 		g_value_set_boolean (value, priv->use_proxy);
 		break;
-#ifdef HAVE_TELEPATHY
-	case PROP_PROTOCOL:
-		g_value_set_string (value, priv->protocol);
-		break;
-	case PROP_CMGR_NAME:
-		g_value_set_string (value, priv->cmgr_name);
-		break;
-#endif
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
 		break;
@@ -302,16 +263,6 @@ account_set_property (GObject      *object,
 		gossip_account_set_use_proxy (GOSSIP_ACCOUNT (object),
 					    g_value_get_boolean (value));
 		break;
-#ifdef HAVE_TELEPATHY
-	case PROP_PROTOCOL:
-		gossip_account_set_protocol (GOSSIP_ACCOUNT (object),
-					     g_value_get_string (value));
-		break;
-	case PROP_CMGR_NAME:
-		gossip_account_set_cmgr_name (GOSSIP_ACCOUNT (object),
-					      g_value_get_string (value));
-		break;
-#endif
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
 		break;
@@ -324,19 +275,24 @@ account_param_new_valist (GossipAccount *account,
 			  va_list        var_args)
 {
 	GossipAccountPriv *priv;
-	const gchar       *name;
+	GossipAccountType  account_type;
+	const gchar       *param_name;
 
 	priv = GET_PRIV (account);
 
-	for (name = first_param_name; name; name = va_arg (var_args, gchar*)) {
+	account_type = gossip_account_get_type (account);
+
+	for (param_name = first_param_name; 
+	     param_name; 
+	     param_name = va_arg (var_args, gchar*)) {
 		GossipAccountParam *param;
 		GType               g_type;
 		gchar              *error = NULL;
 
-		name = account_param_name_convert (name);
-		param = g_hash_table_lookup (priv->parameters, name);
+		param_name = account_param_name_convert (account_type, param_name);
+		param = g_hash_table_lookup (priv->parameters, param_name);
 		if (param) {
-			g_warning ("GossipAccount already has a parameter named `%s'", name);
+			g_warning ("GossipAccount already has a parameter named `%s'", param_name);
 			break;
 		}
 
@@ -355,7 +311,7 @@ account_param_new_valist (GossipAccount *account,
 		param->flags = va_arg (var_args, GossipAccountParamFlags);
 
 		g_hash_table_insert (priv->parameters,
-				     g_strdup (name),
+				     g_strdup (param_name),
 				     param);
 	}
 }
@@ -366,19 +322,24 @@ account_param_set_valist (GossipAccount *account,
 			  va_list        var_args)
 {
 	GossipAccountPriv *priv;
-	const gchar       *name;
+	GossipAccountType  account_type;
+	const gchar       *param_name;
 
 	priv = GET_PRIV (account);
 
-	for (name = first_param_name; name; name = va_arg (var_args, gchar*)) {
+	account_type = gossip_account_get_type (account);
+
+	for (param_name = first_param_name; 
+	     param_name; 
+	     param_name = va_arg (var_args, gchar*)) {
 		GossipAccountParam *param;
 		gchar              *error = NULL;
 		GValue              g_value = {0, };
 
-		name = account_param_name_convert (name);
-		param = g_hash_table_lookup (priv->parameters, name);
+		param_name = account_param_name_convert (account_type, param_name);
+		param = g_hash_table_lookup (priv->parameters, param_name);
 		if (!param) {
-			g_warning ("GossipAccount has no parameter named `%s'", name);
+			g_warning ("GossipAccount has no parameter named `%s'", param_name);
 			break;
 		}
 
@@ -404,15 +365,18 @@ account_param_get_valist (GossipAccount *account,
 			  va_list        var_args)
 {
 	GossipAccountPriv *priv;
+	GossipAccountType  account_type;
 	const gchar       *name;
 
 	priv = GET_PRIV (account);
 
+	account_type = gossip_account_get_type (account);
+
 	for (name = first_param_name; name; name = va_arg (var_args, gchar*)) {
 		GossipAccountParam *param;
 		gchar              *error = NULL;
 
-		name = account_param_name_convert (name);
+		name = account_param_name_convert (account_type, name);
 		param = g_hash_table_lookup (priv->parameters, name);
 		if (!param) {
 			g_warning ("GossipAccount has no parameter named `%s'", name);
@@ -429,7 +393,7 @@ account_param_get_valist (GossipAccount *account,
 }
 
 void
-gossip_account_param_new (GossipAccount *account,
+gossip_account_new_param (GossipAccount *account,
 			  const gchar   *first_param_name,
 			  ...)
 {
@@ -443,35 +407,40 @@ gossip_account_param_new (GossipAccount *account,
 }
 
 void
-gossip_account_param_new_g_value (GossipAccount           *account,
+gossip_account_new_param_g_value (GossipAccount           *account,
 				  const gchar             *param_name,
 				  const GValue            *g_value,
 				  GossipAccountParamFlags  flags)
 {
 	GossipAccountParam *param;
 	GossipAccountPriv  *priv;
+	GossipAccountType   account_type;
+	const gchar        *param_name_converted;
 
 	priv = GET_PRIV (account);
 
-	param_name = account_param_name_convert (param_name);
-	param = g_hash_table_lookup (priv->parameters, param_name);
+	account_type = gossip_account_get_type (account);
+
+	param_name_converted = account_param_name_convert (account_type, param_name);
+	param = g_hash_table_lookup (priv->parameters, param_name_converted);
 	if (param) {
-		g_warning ("GossipAccount already has a parameter named `%s'", param_name);
+		g_warning ("GossipAccount already has a parameter named `%s'", 
+			   param_name_converted);
 		return;
 	}
-	param = g_new0 (GossipAccountParam, 1);
 
+	param = g_new0 (GossipAccountParam, 1);
 	g_value_init (&param->g_value, G_VALUE_TYPE (g_value));
 	g_value_copy (g_value, &param->g_value);
 	param->flags = flags;
 
 	g_hash_table_insert (priv->parameters,
-			     g_strdup (param_name),
+			     g_strdup (param_name_converted),
 			     param);
 }
 
 void
-gossip_account_param_set (GossipAccount *account,
+gossip_account_set_param (GossipAccount *account,
 			  const gchar   *first_param_name,
 			  ...)
 {
@@ -485,18 +454,21 @@ gossip_account_param_set (GossipAccount *account,
 }
 
 void
-gossip_account_param_set_g_value (GossipAccount *account,
+gossip_account_set_param_g_value (GossipAccount *account,
 				  const gchar   *param_name,
 				  const GValue  *g_value)
 {
 	GossipAccountPriv  *priv;
 	GossipAccountParam *param;
+	GossipAccountType   account_type;
 
 	g_return_if_fail (GOSSIP_IS_ACCOUNT (account));
 
 	priv = GET_PRIV (account);
 
-	param_name = account_param_name_convert (param_name);
+	account_type = gossip_account_get_type (account);
+
+	param_name = account_param_name_convert (account_type, param_name);
 	param = g_hash_table_lookup (priv->parameters, param_name);
 	if (!param) {
 		g_warning ("GossipAccount has no parameter named `%s'", param_name);
@@ -510,7 +482,7 @@ gossip_account_param_set_g_value (GossipAccount *account,
 }
 
 void
-gossip_account_param_get (GossipAccount *account,
+gossip_account_get_param (GossipAccount *account,
 			  const gchar   *first_param_name,
 			  ...)
 {
@@ -524,17 +496,20 @@ gossip_account_param_get (GossipAccount *account,
 }
 
 const GValue *
-gossip_account_param_get_g_value (GossipAccount *account,
+gossip_account_get_param_g_value (GossipAccount *account,
 				  const gchar   *param_name)
 {
 	GossipAccountPriv  *priv;
 	GossipAccountParam *param;
+	GossipAccountType   account_type;
 
 	g_return_val_if_fail (GOSSIP_IS_ACCOUNT (account), NULL);
 
 	priv = GET_PRIV (account);
 
-	param_name = account_param_name_convert (param_name);
+	account_type = gossip_account_get_type (account);
+	
+	param_name = account_param_name_convert (account_type, param_name);
 	param = g_hash_table_lookup (priv->parameters, param_name);
 	if (!param) {
 		g_warning ("GossipAccount has no parameter named `%s'", param_name);
@@ -548,45 +523,54 @@ gboolean
 gossip_account_has_param (GossipAccount *account,
 			  const gchar   *param_name)
 {
-	GossipAccountPriv      *priv;
+	GossipAccountPriv *priv;
+	GossipAccountType  account_type;
 
 	g_return_val_if_fail (GOSSIP_IS_ACCOUNT (account), FALSE);
 
 	priv = GET_PRIV (account);
 
-	param_name = account_param_name_convert (param_name);
+	account_type = gossip_account_get_type (account);
+	param_name = account_param_name_convert (account_type, param_name);
+
 	return g_hash_table_lookup (priv->parameters, param_name) != NULL;
 }
 
 static void
-account_param_get_all_foreach (gchar               *param_name,
-			       GossipAccountParam  *param,
-			       GList              **params)
+account_param_get_all_foreach (gchar              *param_name,
+			       GossipAccountParam *param,
+			       GetParamsData      *data)
 {
-	*params = g_list_prepend (*params, g_strdup (account_param_name_convert (param_name)));
+	const gchar *final_param_name;
+
+	final_param_name = account_param_name_convert (data->account_type, param_name);
+	data->params = g_list_prepend (data->params, g_strdup (final_param_name));
 }
 
 GList *
-gossip_account_param_get_all (GossipAccount *account)
+gossip_account_get_param_all (GossipAccount *account)
 {
 	GossipAccountPriv *priv;
-	GList             *params = NULL;
+	GetParamsData      data;
 
 	g_return_val_if_fail (GOSSIP_IS_ACCOUNT (account), NULL);
 
 	priv = GET_PRIV (account);
 
+	data.account_type = gossip_account_get_type (account);
+	data.params = NULL;
+
 	g_hash_table_foreach (priv->parameters,
 			      (GHFunc) account_param_get_all_foreach,
-			      &params);
+			      &data);
 
-	params = g_list_sort (params, (GCompareFunc) strcmp);
+	data.params = g_list_sort (data.params, (GCompareFunc) strcmp);
 
-	return params;
+	return data.params;
 }
 
 GossipAccountParam *
-gossip_account_param_get_param (GossipAccount *account,
+gossip_account_get_param_param (GossipAccount *account,
 				const gchar   *param_name)
 {
 	GossipAccountPriv *priv;
@@ -599,39 +583,6 @@ gossip_account_param_get_param (GossipAccount *account,
 }
 
 
-static void
-account_param_foreach_foreach (const gchar            *param_name,
-			       GossipAccountParam     *param,
-			       GossipAccountParamData *data)
-{
-	data->callback (data->account,
-			account_param_name_convert (param_name),
-			param,
-			data->user_data);
-}
-
-void
-gossip_account_param_foreach (GossipAccount           *account,
-			      GossipAccountParamFunc   callback,
-			      gpointer                 user_data)
-{
-	GossipAccountPriv      *priv;
-	GossipAccountParamData *data;
-
-	g_return_if_fail (GOSSIP_IS_ACCOUNT (account));
-
-	priv = GET_PRIV (account);
-
-	data = g_new0 (GossipAccountParamData, 1);
-	data->callback = callback;
-	data->user_data = user_data;
-	data->account = account;
-
-	g_hash_table_foreach (priv->parameters,
-			      (GHFunc) account_param_foreach_foreach,
-			      data);
-}
-
 static void
 account_param_free (GossipAccountParam *param)
 {
@@ -639,6 +590,23 @@ account_param_free (GossipAccountParam *param)
 	g_slice_free (GossipAccountParam, param);
 }
 
+const gchar *
+account_param_name_convert (GossipAccountType  type,
+			    const gchar       *param_name)
+{
+	if (type == GOSSIP_ACCOUNT_TYPE_JABBER_LEGACY) {
+		return param_name;
+	} 
+
+#ifdef HAVE_TELEPATHY
+	if (strcmp (param_name, "use_ssl") == 0) {
+		return "old-ssl";
+	}
+#endif
+
+	return param_name;
+}
+
 GossipAccountType
 gossip_account_get_type (GossipAccount *account)
 {
@@ -740,64 +708,6 @@ gossip_account_set_use_proxy (GossipAccount *account,
 	g_object_notify (G_OBJECT (account), "use_proxy");
 }
 
-#ifdef HAVE_TELEPATHY
-const gchar *
-gossip_account_get_protocol (GossipAccount *account)
-{
-	GossipAccountPriv *priv;
-
-	g_return_val_if_fail (GOSSIP_IS_ACCOUNT (account), NULL);
-
-	priv = GET_PRIV (account);
-	return priv->protocol;
-}
-
-const gchar *
-gossip_account_get_cmgr_name (GossipAccount *account)
-{
-	GossipAccountPriv *priv;
-
-	g_return_val_if_fail (GOSSIP_IS_ACCOUNT (account), NULL);
-
-	priv = GET_PRIV (account);
-	return priv->cmgr_name;
-}
-
-void 
-gossip_account_set_protocol (GossipAccount *account,
-			     const gchar   *protocol)
-{
-	GossipAccountPriv *priv;
-
-	g_return_if_fail (GOSSIP_IS_ACCOUNT (account));
-	g_return_if_fail (protocol != NULL);
-	
-	priv = GET_PRIV (account);
-	
-	g_free (priv->protocol);
-	priv->protocol = g_strdup (protocol);
-
-	g_object_notify (G_OBJECT (account), "protocol");
-}
-
-void 
-gossip_account_set_cmgr_name (GossipAccount *account,
-			 const gchar   *cmgr_name)
-{
-	GossipAccountPriv *priv;
-
-	g_return_if_fail (GOSSIP_IS_ACCOUNT (account));
-	g_return_if_fail (cmgr_name != NULL);
-	
-	priv = GET_PRIV (account);
-	
-	g_free (priv->cmgr_name);
-	priv->cmgr_name = g_strdup (cmgr_name);
-
-	g_object_notify (G_OBJECT (account), "cmgr_name");
-}
-#endif
-
 guint
 gossip_account_hash (gconstpointer key)
 {
@@ -815,13 +725,15 @@ const gchar *
 gossip_account_type_to_string (GossipAccountType type)
 {
 	switch (type) {
-	case GOSSIP_ACCOUNT_TYPE_JABBER:  return "Jabber";
-	case GOSSIP_ACCOUNT_TYPE_AIM:     return "AIM";
-	case GOSSIP_ACCOUNT_TYPE_ICQ:     return "ICQ";
-	case GOSSIP_ACCOUNT_TYPE_MSN:     return "MSN";
-	case GOSSIP_ACCOUNT_TYPE_YAHOO:   return "Yahoo!";
-
-	case GOSSIP_ACCOUNT_TYPE_UNKNOWN:
+	case GOSSIP_ACCOUNT_TYPE_JABBER_LEGACY:  return "Jabber";
+	case GOSSIP_ACCOUNT_TYPE_JABBER:         return "Experimental: Jabber";
+	case GOSSIP_ACCOUNT_TYPE_AIM:            return "Experimental: AIM";
+	case GOSSIP_ACCOUNT_TYPE_ICQ:            return "Experimental: ICQ";
+	case GOSSIP_ACCOUNT_TYPE_MSN:            return "Experimental: MSN";
+	case GOSSIP_ACCOUNT_TYPE_YAHOO:          return "Experimental: Yahoo!";
+	case GOSSIP_ACCOUNT_TYPE_IRC:            return "Experimental: IRC";
+	case GOSSIP_ACCOUNT_TYPE_SALUT:          return "Experimental: Salut";
+       	case GOSSIP_ACCOUNT_TYPE_UNKNOWN:
 	default:
 		break;
 	}
@@ -833,32 +745,132 @@ GossipAccountType
 gossip_account_string_to_type (const gchar *str)
 {
 	if (gossip_strncasecmp (str, "Jabber", -1) == 0) {
+		return GOSSIP_ACCOUNT_TYPE_JABBER_LEGACY;
+	}
+	else if (gossip_strncasecmp (str, "Experimental: Jabber", -1) == 0) {
 		return GOSSIP_ACCOUNT_TYPE_JABBER;
 	}
-	else if (gossip_strncasecmp (str, "AIM", -1) == 0) {
+	else if (gossip_strncasecmp (str, "Experimental: AIM", -1) == 0) {
 		return GOSSIP_ACCOUNT_TYPE_AIM;
 	}
-	else if (gossip_strncasecmp (str, "ICQ", -1) == 0) {
+	else if (gossip_strncasecmp (str, "Experimental: ICQ", -1) == 0) {
 		return GOSSIP_ACCOUNT_TYPE_ICQ;
 	}
-	else if (gossip_strncasecmp (str, "MSN", -1) == 0) {
+	else if (gossip_strncasecmp (str, "Experimental: MSN", -1) == 0) {
 		return GOSSIP_ACCOUNT_TYPE_MSN;
 	}
-	else if (gossip_strncasecmp (str, "Yahoo!", -1) == 0) {
+	else if (gossip_strncasecmp (str, "Experimental: Yahoo!", -1) == 0) {
 		return GOSSIP_ACCOUNT_TYPE_YAHOO;
 	}
+	else if (gossip_strncasecmp (str, "Experimental: IRC", -1) == 0) {
+		return GOSSIP_ACCOUNT_TYPE_IRC;
+	} else if (gossip_strncasecmp (str, "Experimental: Salut", -1) == 0) {
+		return GOSSIP_ACCOUNT_TYPE_SALUT;
+	}
 
 	return GOSSIP_ACCOUNT_TYPE_UNKNOWN;
 }
 
-const gchar *
-account_param_name_convert (const gchar *param_name)
+GdkPixbuf *
+gossip_account_type_create_pixbuf (GossipAccountType    type,
+				   GtkIconSize          icon_size)
 {
-#ifdef HAVE_TELEPATHY
-	if (strcmp (param_name, "use_ssl") == 0) {
-		return "old-ssl";
+	GtkIconTheme  *theme;
+	GdkPixbuf     *pixbuf = NULL;
+	GError        *error = NULL;
+	gint           w, h;
+	gint           size = 48;
+	const gchar   *icon_id = NULL;
+
+	theme = gtk_icon_theme_get_default ();
+
+	if (!gtk_icon_size_lookup (icon_size, &w, &h)) {
+		size = 48;
+	} else {
+		size = (w + h) / 2;
 	}
-#endif
-	return param_name;
+
+	switch (type) {
+	case GOSSIP_ACCOUNT_TYPE_JABBER_LEGACY:
+	case GOSSIP_ACCOUNT_TYPE_JABBER:
+		icon_id = "im-jabber";
+		break;
+	case GOSSIP_ACCOUNT_TYPE_AIM:
+		icon_id = "im-aim";
+		break;
+	case GOSSIP_ACCOUNT_TYPE_ICQ:
+		icon_id = "im-icq";
+		break;
+	case GOSSIP_ACCOUNT_TYPE_MSN:
+		icon_id = "im-msn";
+		break;
+	case GOSSIP_ACCOUNT_TYPE_YAHOO:
+		icon_id = "im-yahoo";
+		break;
+
+	/* FIXME: we should have an artwork for these protocols */
+	case GOSSIP_ACCOUNT_TYPE_IRC:
+	case GOSSIP_ACCOUNT_TYPE_SALUT:
+	case GOSSIP_ACCOUNT_TYPE_UNKNOWN:
+	default:
+		icon_id = "im";
+		break;
+	}
+
+	pixbuf = gtk_icon_theme_load_icon (theme,
+					   icon_id,     /* Icon name */
+					   size,        /* Size */
+					   0,           /* Flags */
+					   &error);
+
+	return pixbuf;
+}
+
+GdkPixbuf *
+gossip_account_create_pixbuf (GossipAccount       *account,
+			      GtkIconSize          icon_size)
+{
+	GossipAccountType  type;
+	GdkPixbuf         *pixbuf = NULL;
+
+	g_return_val_if_fail (GOSSIP_IS_ACCOUNT (account), NULL);
+
+	type = gossip_account_get_type (account);
+	pixbuf = gossip_account_type_create_pixbuf (type, icon_size);
+
+	return pixbuf;
+}
+
+GdkPixbuf *
+gossip_account_status_create_pixbuf (GossipAccount       *account,
+				     GtkIconSize          icon_size,
+				     gboolean             online)
+{
+	GdkPixbuf *pixbuf = NULL;
+
+	g_return_val_if_fail (GOSSIP_IS_ACCOUNT (account), NULL);
+
+	pixbuf = gossip_account_create_pixbuf (account, icon_size);
+	g_return_val_if_fail (pixbuf != NULL, NULL);
+
+	if (!online) {
+		GdkPixbuf *modded_pixbuf;
+
+		modded_pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB,
+						TRUE,
+						8,
+						gdk_pixbuf_get_width (pixbuf),
+						gdk_pixbuf_get_height (pixbuf));
+
+		gdk_pixbuf_saturate_and_pixelate (pixbuf,
+						  modded_pixbuf,
+						  1.0,
+						  TRUE);
+		g_object_unref (pixbuf);
+		pixbuf = modded_pixbuf;
+	}
+
+	return pixbuf;
+
 }
 
