@@ -16,36 +16,40 @@
  * License along with this program; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
+ *
+ * Authors: Mikael Hallendal <micke@imendio.com>
  */
 
-#include <config.h>
+#include "config.h"
 
 #include <glib/gi18n.h>
 
-/* FIXME: we should really have a definition in config.h so we can
- * include certain protocol head files.
- */
-#include <gossip-jabber.h>
+#ifdef HAVE_TELEPATHY
+#include <protocols/telepathy/gossip-telepathy.h>
+#endif
+
+#include <protocols/jabber/gossip-jabber.h>
 
-#include "libgossip-marshal.h"
 #include "gossip-protocol.h"
 
-static void    gossip_protocol_class_init       (GossipProtocolClass *klass);
-static void    gossip_protocol_init             (GossipProtocol      *protocol);
+#include "libgossip-marshal.h"
+
+static void gossip_protocol_class_init (GossipProtocolClass *klass);
+static void gossip_protocol_init       (GossipProtocol      *protocol);
 
 enum {
-	LOGGED_IN,
-	LOGGED_OUT,
+	CONNECTING,
+	CONNECTED,
+	DISCONNECTING,
+	DISCONNECTED,
 	NEW_MESSAGE,
 	CONTACT_ADDED,
-	CONTACT_UPDATED,
-	CONTACT_PRESENCE_UPDATED,
 	CONTACT_REMOVED,
 	COMPOSING,
 
 	/* Used to get password from user. */
 	GET_PASSWORD,
-	
+
 	SUBSCRIPTION_REQUEST,
 
 	ERROR,
@@ -74,42 +78,62 @@ gossip_protocol_class_init (GossipProtocolClass *klass)
 	klass->set_vcard            = NULL;
 	klass->send_message         = NULL;
 	klass->send_composing       = NULL;
-        klass->find_contact         = NULL;
-        klass->add_contact          = NULL;
-        klass->rename_contact       = NULL;
-        klass->remove_contact       = NULL;
+	klass->find_contact         = NULL;
+	klass->add_contact          = NULL;
+	klass->rename_contact       = NULL;
+	klass->remove_contact       = NULL;
 	klass->update_contact       = NULL;
 	klass->rename_group         = NULL;
 	klass->get_contacts         = NULL;
 	klass->get_own_contact      = NULL;
 	klass->get_active_resource  = NULL;
 	klass->get_groups           = NULL;
- 	klass->get_vcard            = NULL;
- 	klass->get_version          = NULL;
+	klass->get_vcard            = NULL;
+	klass->get_version          = NULL;
 	klass->register_account     = NULL;
- 
-	signals[LOGGED_IN] = 
-		g_signal_new ("logged-in",
+	klass->new_account          = NULL;
+
+	signals[CONNECTING] =
+		g_signal_new ("connecting",
 			      G_TYPE_FROM_CLASS (klass),
 			      G_SIGNAL_RUN_LAST,
 			      0,
 			      NULL, NULL,
 			      libgossip_marshal_VOID__OBJECT,
-			      G_TYPE_NONE, 
+			      G_TYPE_NONE,
 			      1, GOSSIP_TYPE_ACCOUNT);
 
-	/* Maybe include a reason for disconnect? */
-	signals[LOGGED_OUT] = 
-		g_signal_new ("logged-out",
+	signals[CONNECTED] =
+		g_signal_new ("connected",
+			      G_TYPE_FROM_CLASS (klass),
+			      G_SIGNAL_RUN_LAST,
+			      0,
+			      NULL, NULL,
+			      libgossip_marshal_VOID__OBJECT,
+			      G_TYPE_NONE,
+			      1, GOSSIP_TYPE_ACCOUNT);
+
+	signals[DISCONNECTING] =
+		g_signal_new ("disconnecting",
+			      G_TYPE_FROM_CLASS (klass),
+			      G_SIGNAL_RUN_LAST,
+			      0,
+			      NULL, NULL,
+			      libgossip_marshal_VOID__OBJECT,
+			      G_TYPE_NONE,
+			      1, GOSSIP_TYPE_ACCOUNT);
+
+	signals[DISCONNECTED] =
+		g_signal_new ("disconnected",
 			      G_TYPE_FROM_CLASS (klass),
 			      G_SIGNAL_RUN_LAST,
 			      0,
 			      NULL, NULL,
 			      libgossip_marshal_VOID__OBJECT_INT,
-			      G_TYPE_NONE, 
+			      G_TYPE_NONE,
 			      2, GOSSIP_TYPE_ACCOUNT, G_TYPE_INT);
 
-	signals[NEW_MESSAGE] = 
+	signals[NEW_MESSAGE] =
 		g_signal_new ("new-message",
 			      G_TYPE_FROM_CLASS (klass),
 			      G_SIGNAL_RUN_LAST,
@@ -119,7 +143,7 @@ gossip_protocol_class_init (GossipProtocolClass *klass)
 			      G_TYPE_NONE,
 			      1, GOSSIP_TYPE_MESSAGE);
 
-	signals[CONTACT_ADDED] = 
+	signals[CONTACT_ADDED] =
 		g_signal_new ("contact-added",
 			      G_TYPE_FROM_CLASS (klass),
 			      G_SIGNAL_RUN_LAST,
@@ -129,27 +153,7 @@ gossip_protocol_class_init (GossipProtocolClass *klass)
 			      G_TYPE_NONE,
 			      1, GOSSIP_TYPE_CONTACT);
 
-	signals[CONTACT_UPDATED] = 
-		g_signal_new ("contact-updated",
-			      G_TYPE_FROM_CLASS (klass),
-			      G_SIGNAL_RUN_LAST,
-			      0,
-			      NULL, NULL,
-			      libgossip_marshal_VOID__OBJECT,
-			      G_TYPE_NONE,
-			      1, GOSSIP_TYPE_CONTACT);
-	
-	signals[CONTACT_PRESENCE_UPDATED] =
-		g_signal_new ("contact-presence-updated",
-			      G_TYPE_FROM_CLASS (klass),
-			      G_SIGNAL_RUN_LAST,
-			      0,
-			      NULL, NULL,
-			      libgossip_marshal_VOID__OBJECT,
-			      G_TYPE_NONE,
-			      1, GOSSIP_TYPE_CONTACT);
-
-	signals[CONTACT_REMOVED] = 
+	signals[CONTACT_REMOVED] =
 		g_signal_new ("contact-removed",
 			      G_TYPE_FROM_CLASS (klass),
 			      G_SIGNAL_RUN_LAST,
@@ -159,7 +163,7 @@ gossip_protocol_class_init (GossipProtocolClass *klass)
 			      G_TYPE_NONE,
 			      1, GOSSIP_TYPE_CONTACT);
 
-	signals[COMPOSING] = 
+	signals[COMPOSING] =
 		g_signal_new ("composing",
 			      G_TYPE_FROM_CLASS (klass),
 			      G_SIGNAL_RUN_LAST,
@@ -179,17 +183,17 @@ gossip_protocol_class_init (GossipProtocolClass *klass)
 			      G_TYPE_STRING,
 			      1, GOSSIP_TYPE_ACCOUNT);
 
-        signals[SUBSCRIPTION_REQUEST] =
-                g_signal_new ("subscription-request",
+	signals[SUBSCRIPTION_REQUEST] =
+		g_signal_new ("subscription-request",
 			      G_TYPE_FROM_CLASS (klass),
 			      G_SIGNAL_RUN_LAST,
 			      0,
 			      NULL, NULL,
-                              libgossip_marshal_VOID__OBJECT,
+			      libgossip_marshal_VOID__OBJECT,
 			      G_TYPE_NONE,
 			      1, GOSSIP_TYPE_CONTACT);
 
-	signals[ERROR] = 
+	signals[ERROR] =
 		g_signal_new ("error",
 			      G_TYPE_FROM_CLASS (klass),
 			      G_SIGNAL_RUN_LAST,
@@ -212,18 +216,51 @@ gossip_protocol_new_from_account_type (GossipAccountType type)
 {
 	GossipProtocol *protocol = NULL;
 
-	/* create protocol for account type */
-	switch (type) {
-	case GOSSIP_ACCOUNT_TYPE_JABBER:
+	if (type == GOSSIP_ACCOUNT_TYPE_JABBER_LEGACY) {
 		protocol = g_object_new (GOSSIP_TYPE_JABBER, NULL);
-		break;
-	default:
-		break;
+#ifdef HAVE_TELEPATHY
+	} else {
+
+		protocol = g_object_new (GOSSIP_TYPE_TELEPATHY, NULL);
+#endif
 	}
 
 	return protocol;
 }
 
+GossipAccount *
+gossip_protocol_new_account (GossipProtocol    *protocol,
+			     GossipAccountType  type)
+{
+	GossipProtocolClass *klass;
+
+	g_return_val_if_fail (GOSSIP_IS_PROTOCOL (protocol), NULL);
+
+	klass = GOSSIP_PROTOCOL_GET_CLASS (protocol);
+	if (klass->new_account) {
+		return klass->new_account (protocol, type);
+	}
+
+	return NULL;
+}
+
+GossipContact *
+gossip_protocol_new_contact (GossipProtocol *protocol,
+			     const gchar    *id,
+			     const gchar    *name)
+{
+	GossipProtocolClass *klass;
+
+	g_return_val_if_fail (GOSSIP_IS_PROTOCOL (protocol), NULL);
+
+	klass = GOSSIP_PROTOCOL_GET_CLASS (protocol);
+	if (klass->new_contact) {
+		return klass->new_contact (protocol, id, name);
+	}
+
+	return NULL;
+}
+
 void
 gossip_protocol_setup (GossipProtocol *protocol,
 		       GossipAccount  *account)
@@ -239,7 +276,7 @@ gossip_protocol_setup (GossipProtocol *protocol,
 	}
 }
 
-void 
+void
 gossip_protocol_login (GossipProtocol *protocol)
 {
 	GossipProtocolClass *klass;
@@ -327,7 +364,7 @@ gossip_protocol_is_ssl_supported (GossipProtocol *protocol)
 	return FALSE;
 }
 
-const gchar * 
+const gchar *
 gossip_protocol_get_example_username (GossipProtocol *protocol)
 {
 	GossipProtocolClass *klass;
@@ -342,7 +379,7 @@ gossip_protocol_get_example_username (GossipProtocol *protocol)
 	return NULL;
 }
 
-gchar * 
+gchar *
 gossip_protocol_get_default_server (GossipProtocol *protocol,
 				    const gchar    *username)
 {
@@ -359,7 +396,7 @@ gossip_protocol_get_default_server (GossipProtocol *protocol,
 	return NULL;
 }
 
-guint16 
+guint
 gossip_protocol_get_default_port (GossipProtocol *protocol,
 				  gboolean        use_ssl)
 {
@@ -376,7 +413,7 @@ gossip_protocol_get_default_port (GossipProtocol *protocol,
 }
 
 void
-gossip_protocol_send_message (GossipProtocol *protocol, 
+gossip_protocol_send_message (GossipProtocol *protocol,
 			      GossipMessage  *message)
 {
 	GossipProtocolClass *klass;
@@ -405,7 +442,7 @@ gossip_protocol_send_composing (GossipProtocol *protocol,
 }
 
 void
-gossip_protocol_set_presence (GossipProtocol *protocol, 
+gossip_protocol_set_presence (GossipProtocol *protocol,
 			      GossipPresence *presence)
 {
 	GossipProtocolClass *klass;
@@ -433,12 +470,12 @@ gossip_protocol_set_subscription (GossipProtocol *protocol,
 	}
 }
 
-gboolean 
-gossip_protocol_set_vcard (GossipProtocol        *protocol,
-			   GossipVCard           *vcard,
-			   GossipResultCallback   callback,
-			   gpointer               user_data,
-			   GError               **error)
+gboolean
+gossip_protocol_set_vcard (GossipProtocol  *protocol,
+			   GossipVCard     *vcard,
+			   GossipCallback   callback,
+			   gpointer         user_data,
+			   GError         **error)
 {
 	GossipProtocolClass *klass;
 
@@ -447,8 +484,8 @@ gossip_protocol_set_vcard (GossipProtocol        *protocol,
 	klass = GOSSIP_PROTOCOL_GET_CLASS (protocol);
 	if (klass->set_vcard) {
 		return klass->set_vcard (protocol, vcard,
-					       callback, user_data, 
-					       error);
+					 callback, user_data,
+					 error);
 	}
 
 	/* Don't report error if protocol doesn't implement this */
@@ -459,39 +496,39 @@ GossipContact *
 gossip_protocol_find_contact (GossipProtocol *protocol,
 			      const gchar    *id)
 {
-        GossipProtocolClass *klass;
+	GossipProtocolClass *klass;
 
 	g_return_val_if_fail (GOSSIP_IS_PROTOCOL (protocol), NULL);
 
 	klass = GOSSIP_PROTOCOL_GET_CLASS (protocol);
 	if (klass->find_contact) {
-                return klass->find_contact (protocol, id);
-        }
+		return klass->find_contact (protocol, id);
+	}
 
 	return NULL;
 }
 
 void
 gossip_protocol_add_contact (GossipProtocol *protocol,
-                             const gchar    *id,
-                             const gchar    *name,
-                             const gchar    *group,
-                             const gchar    *message)
+			     const gchar    *id,
+			     const gchar    *name,
+			     const gchar    *group,
+			     const gchar    *message)
 {
-        GossipProtocolClass *klass;
+	GossipProtocolClass *klass;
 
 	g_return_if_fail (GOSSIP_IS_PROTOCOL (protocol));
 
 	klass = GOSSIP_PROTOCOL_GET_CLASS (protocol);
 	if (klass->add_contact) {
-                klass->add_contact (protocol, id, name, group, message);
-        }
+		klass->add_contact (protocol, id, name, group, message);
+	}
 }
 
 void
 gossip_protocol_rename_contact (GossipProtocol *protocol,
-                                GossipContact  *contact,
-                                const gchar    *new_name)
+				GossipContact  *contact,
+				const gchar    *new_name)
 {
 	GossipProtocolClass *klass;
 
@@ -505,7 +542,7 @@ gossip_protocol_rename_contact (GossipProtocol *protocol,
 
 void
 gossip_protocol_remove_contact (GossipProtocol *protocol,
-                                GossipContact  *contact)
+				GossipContact  *contact)
 {
 	GossipProtocolClass *klass;
 
@@ -519,7 +556,7 @@ gossip_protocol_remove_contact (GossipProtocol *protocol,
 
 void
 gossip_protocol_update_contact (GossipProtocol *protocol,
-                                GossipContact  *contact)
+				GossipContact  *contact)
 {
 	GossipProtocolClass *klass;
 
@@ -546,7 +583,7 @@ gossip_protocol_rename_group (GossipProtocol *protocol,
 	}
 }
 
-GossipContact * 
+GossipContact *
 gossip_protocol_get_own_contact (GossipProtocol *protocol)
 {
 	GossipProtocolClass *klass;
@@ -576,7 +613,7 @@ gossip_protocol_get_contacts (GossipProtocol *protocol)
 	return NULL;
 }
 
-const gchar * 
+const gchar *
 gossip_protocol_get_active_resource (GossipProtocol *protocol,
 				     GossipContact  *contact)
 {
@@ -592,7 +629,7 @@ gossip_protocol_get_active_resource (GossipProtocol *protocol,
 	return NULL;
 }
 
-GList * 
+GList *
 gossip_protocol_get_groups (GossipProtocol *protocol)
 {
 	GossipProtocolClass *klass;
@@ -621,7 +658,7 @@ gossip_protocol_get_vcard (GossipProtocol       *protocol,
 	klass = GOSSIP_PROTOCOL_GET_CLASS (protocol);
 	if (klass->get_vcard) {
 		return klass->get_vcard (protocol, contact,
-					 callback, user_data, 
+					 callback, user_data,
 					 error);
 	}
 
@@ -643,20 +680,20 @@ gossip_protocol_get_version (GossipProtocol         *protocol,
 	klass = GOSSIP_PROTOCOL_GET_CLASS (protocol);
 	if (klass->get_version) {
 		return klass->get_version (protocol, contact,
-						 callback, user_data, 
+						 callback, user_data,
 						 error);
 	}
 
 	/* Don't report error if protocol doesn't implement this */
 	return TRUE;
 }
- 
+
 void
-gossip_protocol_register_account (GossipProtocol          *protocol,
-				  GossipAccount           *account,
-				  GossipVCard             *vcard,
-				  GossipRegisterCallback   callback,
-				  gpointer                 user_data)
+gossip_protocol_register_account (GossipProtocol      *protocol,
+				  GossipAccount       *account,
+				  GossipVCard         *vcard,
+				  GossipErrorCallback  callback,
+				  gpointer             user_data)
 {
 	GossipProtocolClass *klass;
 
@@ -683,6 +720,61 @@ gossip_protocol_register_cancel (GossipProtocol *protocol)
 	}
 }
 
+void
+gossip_protocol_change_password (GossipProtocol      *protocol,
+				 const gchar         *new_password,
+				 GossipErrorCallback  callback,
+				 gpointer             user_data)
+{
+	GossipProtocolClass *klass;
+
+	g_return_if_fail (GOSSIP_IS_PROTOCOL (protocol));
+	g_return_if_fail (new_password != NULL);
+	g_return_if_fail (callback != NULL);
+
+	klass = GOSSIP_PROTOCOL_GET_CLASS (protocol);
+	if (klass->change_password) {
+		klass->change_password (protocol, new_password,
+					callback, user_data);
+	}
+}
+
+void
+gossip_protocol_change_password_cancel (GossipProtocol *protocol)
+{
+	GossipProtocolClass *klass;
+
+	g_return_if_fail (GOSSIP_IS_PROTOCOL (protocol));
+
+	klass = GOSSIP_PROTOCOL_GET_CLASS (protocol);
+	if (klass->change_password_cancel) {
+		klass->change_password_cancel (protocol);
+	}
+}
+
+void
+gossip_protocol_get_avatar_requirements (GossipProtocol  *protocol,
+					 guint           *min_width,
+					 guint           *min_height,
+					 guint           *max_width,
+					 guint           *max_height,
+					 gsize           *max_size,
+					 gchar          **format)
+{
+	GossipProtocolClass *klass;
+
+	g_return_if_fail (GOSSIP_IS_PROTOCOL (protocol));
+
+	klass = GOSSIP_PROTOCOL_GET_CLASS (protocol);
+	if (klass->get_avatar_requirements) {
+		klass->get_avatar_requirements (protocol,
+						min_width, min_height,
+						max_width, max_height,
+						max_size,
+						format);
+	}
+}
+
 const gchar *
 gossip_protocol_error_to_string (GossipProtocolError error)
 {
