@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2004-2005 Imendio AB
+ * Copyright (C) 2004-2007 Imendio AB
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -16,14 +16,20 @@
  * License along with this program; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
+ *
+ * Authors: Mikael Hallendal <micke@imendio.com>
  */
 
-#include <config.h>
+#include "config.h"
+
 #include <string.h>
 
-#include "libgossip-marshal.h"
-#include "gossip-debug.h"
+#include "gossip-chatroom.h"
 #include "gossip-session.h"
+#include "gossip-debug.h"
+#include "gossip-private.h"
+
+#include "libgossip-marshal.h"
 
 #define DEBUG_DOMAIN "Session"
 
@@ -32,19 +38,22 @@
 typedef struct _GossipSessionPriv  GossipSessionPriv;
 
 struct _GossipSessionPriv {
-	GossipAccountManager *account_manager;
+	GossipAccountManager  *account_manager;
+	GossipContactManager  *contact_manager;
+	GossipChatroomManager *chatroom_manager;
+	GossipLogManager      *log_manager;
 
-	GHashTable           *accounts;
-	GList                *protocols;
+	GHashTable            *accounts;
+	GList                 *protocols;
 
-	GossipPresence       *presence;
+	GossipPresence        *presence;
 
-	GList                *contacts;
+	GList                 *contacts;
 
-	guint                 connected_counter;
-	guint                 connecting_counter;
+	guint                  connected_counter;
+	guint                  connecting_counter;
 
-	GHashTable           *timers; /* connected time */
+	GHashTable            *timers; /* connected time */
 };
 
 typedef struct {
@@ -81,10 +90,16 @@ static void            gossip_session_init                       (GossipSession
 static void            session_finalize                          (GObject              *object);
 static void            session_protocol_signals_setup            (GossipSession        *session,
 								  GossipProtocol       *protocol);
-static void            session_protocol_logged_in                (GossipProtocol       *protocol,
+static void            session_protocol_connecting               (GossipProtocol       *protocol,
 								  GossipAccount        *account,
 								  GossipSession        *session);
-static void            session_protocol_logged_out               (GossipProtocol       *protocol,
+static void            session_protocol_connected                (GossipProtocol       *protocol,
+								  GossipAccount        *account,
+								  GossipSession        *session);
+static void            session_protocol_disconnecting            (GossipProtocol       *protocol,
+								  GossipAccount        *account,
+								  GossipSession        *session);
+static void            session_protocol_disconnected             (GossipProtocol       *protocol,
 								  GossipAccount        *account,
 								  gint                  reason,
 								  GossipSession        *session);
@@ -94,12 +109,6 @@ static void            session_protocol_new_message              (GossipProtocol
 static void            session_protocol_contact_added            (GossipProtocol       *protocol,
 								  GossipContact        *contact,
 								  GossipSession        *session);
-static void            session_protocol_contact_updated          (GossipProtocol       *protocol,
-								  GossipContact        *contact,
-								  GossipSession        *session);
-static void            session_protocol_contact_presence_updated (GossipProtocol       *protocol,
-								  GossipContact        *contact,
-								  GossipSession        *session);
 static void            session_protocol_contact_removed          (GossipProtocol       *protocol,
 								  GossipContact        *contact,
 								  GossipSession        *session);
@@ -138,12 +147,10 @@ enum {
 	PROTOCOL_DISCONNECTING,
 	PROTOCOL_ERROR,
 	NEW_MESSAGE,
-	PRESENCE_CHANGED,
 	CONTACT_ADDED,
-	CONTACT_UPDATED,
-	CONTACT_PRESENCE_UPDATED,
 	CONTACT_REMOVED,
 	COMPOSING,
+	CHATROOM_AUTO_CONNECT,
 
 	/* Used for protocols to retreive information from UI */
 	GET_PASSWORD,
@@ -259,16 +266,6 @@ gossip_session_class_init (GossipSessionClass *klass)
 			      G_TYPE_NONE,
 			      1, GOSSIP_TYPE_MESSAGE);
 
-	signals[PRESENCE_CHANGED] =
-		g_signal_new ("presence-changed",
-			      G_TYPE_FROM_CLASS (klass),
-			      G_SIGNAL_RUN_LAST,
-			      0,
-			      NULL, NULL,
-			      libgossip_marshal_VOID__OBJECT,
-			      G_TYPE_NONE,
-			      1, GOSSIP_TYPE_PRESENCE);
-
 	signals[CONTACT_ADDED] =
 		g_signal_new ("contact-added",
 			      G_TYPE_FROM_CLASS (klass),
@@ -278,25 +275,6 @@ gossip_session_class_init (GossipSessionClass *klass)
 			      libgossip_marshal_VOID__OBJECT,
 			      G_TYPE_NONE,
 			      1, GOSSIP_TYPE_CONTACT);
-	signals[CONTACT_UPDATED] =
-		g_signal_new ("contact-updated",
-			      G_TYPE_FROM_CLASS (klass),
-			      G_SIGNAL_RUN_LAST,
-			      0,
-			      NULL, NULL,
-			      libgossip_marshal_VOID__OBJECT,
-			      G_TYPE_NONE,
-			      1, GOSSIP_TYPE_CONTACT);
-
-	signals[CONTACT_PRESENCE_UPDATED] =
-		g_signal_new ("contact-presence-updated",
-			      G_TYPE_FROM_CLASS (klass),
-			      G_SIGNAL_RUN_LAST,
-			      0,
-			      NULL, NULL,
-			      libgossip_marshal_VOID__OBJECT,
-			      G_TYPE_NONE,
-			      1, GOSSIP_TYPE_CONTACT);
 
 	signals[CONTACT_REMOVED] =
 		g_signal_new ("contact-removed",
@@ -307,6 +285,7 @@ gossip_session_class_init (GossipSessionClass *klass)
 			      libgossip_marshal_VOID__OBJECT,
 			      G_TYPE_NONE,
 			      1, GOSSIP_TYPE_CONTACT);
+
 	signals[COMPOSING] =
 		g_signal_new ("composing",
 			      G_TYPE_FROM_CLASS (klass),
@@ -316,6 +295,19 @@ gossip_session_class_init (GossipSessionClass *klass)
 			      libgossip_marshal_VOID__OBJECT_BOOLEAN,
 			      G_TYPE_NONE,
 			      2, GOSSIP_TYPE_CONTACT, G_TYPE_BOOLEAN);
+
+	signals[CHATROOM_AUTO_CONNECT] =
+		g_signal_new ("chatroom-auto-connect",
+			      G_TYPE_FROM_CLASS (klass),
+			      G_SIGNAL_RUN_LAST,
+			      0,
+			      NULL, NULL,
+			      libgossip_marshal_VOID__OBJECT_OBJECT,
+			      G_TYPE_NONE,
+			      2,
+			      GOSSIP_TYPE_CHATROOM_PROVIDER,
+			      GOSSIP_TYPE_CHATROOM);
+
 	signals[GET_PASSWORD] =
 		g_signal_new ("get-password",
 			      G_TYPE_FROM_CLASS (klass),
@@ -349,7 +341,6 @@ gossip_session_init (GossipSession *session)
 					      gossip_account_equal,
 					      g_object_unref,
 					      (GDestroyNotify)g_timer_destroy);
-
 }
 
 static void
@@ -373,6 +364,10 @@ session_finalize (GObject *object)
 
 	g_hash_table_destroy (priv->timers);
 
+	g_object_unref (priv->log_manager);
+	g_object_unref (priv->chatroom_manager);
+	g_object_unref (priv->contact_manager);
+
 	g_signal_handlers_disconnect_by_func (priv->account_manager,
 					      session_account_added_cb,
 					      object);
@@ -390,11 +385,17 @@ static void
 session_protocol_signals_setup (GossipSession  *session,
 				GossipProtocol *protocol)
 {
-	g_signal_connect (protocol, "logged-in",
-			  G_CALLBACK (session_protocol_logged_in),
+	g_signal_connect (protocol, "connecting",
+			  G_CALLBACK (session_protocol_connecting),
 			  session);
-	g_signal_connect (protocol, "logged-out",
-			  G_CALLBACK (session_protocol_logged_out),
+	g_signal_connect (protocol, "connected",
+			  G_CALLBACK (session_protocol_connected),
+			  session);
+	g_signal_connect (protocol, "disconnecting",
+			  G_CALLBACK (session_protocol_disconnecting),
+			  session);
+	g_signal_connect (protocol, "disconnected",
+			  G_CALLBACK (session_protocol_disconnected),
 			  session);
 	g_signal_connect (protocol, "new_message",
 			  G_CALLBACK (session_protocol_new_message),
@@ -402,12 +403,6 @@ session_protocol_signals_setup (GossipSession  *session,
 	g_signal_connect (protocol, "contact-added",
 			  G_CALLBACK (session_protocol_contact_added),
 			  session);
-	g_signal_connect (protocol, "contact-updated",
-			  G_CALLBACK (session_protocol_contact_updated),
-			  session);
-	g_signal_connect (protocol, "contact-presence-updated",
-			  G_CALLBACK (session_protocol_contact_presence_updated),
-			  session);
 	g_signal_connect (protocol, "contact-removed",
 			  G_CALLBACK (session_protocol_contact_removed),
 			  session);
@@ -423,13 +418,32 @@ session_protocol_signals_setup (GossipSession  *session,
 }
 
 static void
-session_protocol_logged_in (GossipProtocol *protocol,
+session_protocol_connecting (GossipProtocol *protocol,
+			     GossipAccount  *account,
+			     GossipSession  *session)
+{
+	GossipSessionPriv *priv;
+
+	gossip_debug (DEBUG_DOMAIN, 
+		      "Protocol connecting for account:'%s'",
+		      gossip_account_get_name (account));
+
+	priv = GET_PRIV (session);
+
+	g_signal_emit (session, signals[PROTOCOL_CONNECTING], 0, account, protocol);
+
+	priv->connecting_counter++;
+}
+
+static void
+session_protocol_connected (GossipProtocol *protocol,
 			    GossipAccount  *account,
 			    GossipSession  *session)
 {
 	GossipSessionPriv *priv;
+	GList             *chatrooms, *l;
 
-	gossip_debug (DEBUG_DOMAIN, "Protocol logged in");
+	gossip_debug (DEBUG_DOMAIN, "Protocol Connected");
 
 	priv = GET_PRIV (session);
 
@@ -450,19 +464,56 @@ session_protocol_logged_in (GossipProtocol *protocol,
 		/* Before this connect the session was set to be DISCONNECTED */
 		g_signal_emit (session, signals[CONNECTED], 0);
 	}
+
+	/* Chatrooms */
+	gossip_debug (DEBUG_DOMAIN,
+		      "Checking chatroom auto connects...",
+		      gossip_account_get_name (account));
+
+	chatrooms = gossip_chatroom_manager_get_chatrooms (priv->chatroom_manager, account);
+	for (l = chatrooms; l; l = l->next) {
+		GossipChatroom *chatroom;
+
+		chatroom = l->data;
+
+		if (gossip_chatroom_get_auto_connect (chatroom)) {
+			GossipChatroomProvider *provider;
+
+			/* Found in list, it needs to be connected to */
+			provider = gossip_session_get_chatroom_provider (session, account);
+
+			g_signal_emit (session, signals[CHATROOM_AUTO_CONNECT], 0,
+				       provider, chatroom);
+		}
+	}
+
+	g_list_free (chatrooms);
 }
 
 static void
-session_protocol_logged_out (GossipProtocol *protocol,
-			     GossipAccount  *account,
-			     gint            reason,
-			     GossipSession  *session)
+session_protocol_disconnecting (GossipProtocol *protocol,
+				GossipAccount  *account,
+				GossipSession  *session)
+{
+	gossip_debug (DEBUG_DOMAIN, 
+		      "Protocol disconnecting for account:'%s'",
+		      gossip_account_get_name (account));
+	
+	g_signal_emit (session, signals[PROTOCOL_DISCONNECTING], 0, account, protocol);
+}
+
+static void
+session_protocol_disconnected (GossipProtocol *protocol,
+			       GossipAccount  *account,
+			       gint            reason,
+			       GossipSession  *session)
 {
 	GossipSessionPriv *priv;
 	gdouble            seconds;
 
 	seconds = gossip_session_get_connected_time (session, account);
-	gossip_debug (DEBUG_DOMAIN, "Protocol logged out (after %.2f seconds)",
+	gossip_debug (DEBUG_DOMAIN, 
+		      "Protocol disconnected (after %.2f seconds)",
 		      seconds);
 
 	priv = GET_PRIV (session);
@@ -521,33 +572,13 @@ session_protocol_contact_added (GossipProtocol *protocol,
 	g_signal_emit (session, signals[CONTACT_ADDED], 0, contact);
 }
 
-static void
-session_protocol_contact_updated (GossipProtocol *protocol,
-				  GossipContact  *contact,
-				  GossipSession  *session)
-{
-	gossip_debug (DEBUG_DOMAIN, "Contact updated '%s'",
-		      gossip_contact_get_name (contact));
-
-	g_signal_emit (session, signals[CONTACT_UPDATED], 0, contact);
-}
-
-static void
-session_protocol_contact_presence_updated (GossipProtocol *protocol,
-					   GossipContact  *contact,
-					   GossipSession  *session)
-{
-	gossip_debug (DEBUG_DOMAIN, "Contact presence updated '%s'",
-		      gossip_contact_get_name (contact));
-	g_signal_emit (session, signals[CONTACT_PRESENCE_UPDATED], 0, contact);
-}
-
 static void
 session_protocol_contact_removed (GossipProtocol *protocol,
 				  GossipContact  *contact,
 				  GossipSession  *session)
 {
 	GossipSessionPriv *priv;
+	GList             *link;
 
 	gossip_debug (DEBUG_DOMAIN, "Contact removed '%s'",
 		   gossip_contact_get_name (contact));
@@ -556,8 +587,11 @@ session_protocol_contact_removed (GossipProtocol *protocol,
 
 	g_signal_emit (session, signals[CONTACT_REMOVED], 0, contact);
 
-	priv->contacts = g_list_remove (priv->contacts, contact);
-	g_object_unref (contact);
+	link = g_list_find (priv->contacts, contact);
+	if (link) {
+		priv->contacts = g_list_delete_link (priv->contacts, link);
+		g_object_unref (contact);
+	}
 }
 
 static void
@@ -646,19 +680,20 @@ session_get_protocol (GossipSession *session,
 }
 
 GossipSession *
-gossip_session_new (GossipAccountManager *manager)
+gossip_session_new (const gchar *accounts_file,
+		    const gchar *contacts_file,
+		    const gchar *chatrooms_file)
 {
-	GossipSession     *session;
 	GossipSessionPriv *priv;
+	GossipSession     *session;
 	GList             *accounts, *l;
 
-	g_return_val_if_fail (GOSSIP_IS_ACCOUNT_MANAGER (manager), NULL);
-
 	session = g_object_new (GOSSIP_TYPE_SESSION, NULL);
 
 	priv = GET_PRIV (session);
 
-	priv->account_manager = g_object_ref (manager);
+	/* Set up account manager */
+	priv->account_manager = gossip_account_manager_new (accounts_file);
 
 	g_signal_connect (priv->account_manager, "account_added",
 			  G_CALLBACK (session_account_added_cb),
@@ -668,7 +703,8 @@ gossip_session_new (GossipAccountManager *manager)
 			  G_CALLBACK (session_account_removed_cb),
 			  session);
 
-	accounts = gossip_account_manager_get_accounts (manager);
+	/* Set up accounts */
+	accounts = gossip_account_manager_get_accounts (priv->account_manager);
 
 	for (l = accounts; l; l = l->next) {
 		GossipAccount *account = l->data;
@@ -679,6 +715,17 @@ gossip_session_new (GossipAccountManager *manager)
 	g_list_foreach (accounts, (GFunc) g_object_unref, NULL);
 	g_list_free (accounts);
 
+	/* Set up contact manager */
+	priv->contact_manager = gossip_contact_manager_new (session,
+							    contacts_file);
+
+	/* Set up chatroom manager */
+	priv->chatroom_manager = gossip_chatroom_manager_new (priv->account_manager, 
+							      chatrooms_file);
+
+	/* Set up log manager */
+	priv->log_manager = gossip_log_manager_new (session);
+
 	return session;
 }
 
@@ -708,6 +755,42 @@ gossip_session_get_account_manager (GossipSession *session)
 	return priv->account_manager;
 }
 
+GossipContactManager *
+gossip_session_get_contact_manager (GossipSession *session)
+{
+	GossipSessionPriv *priv;
+
+	g_return_val_if_fail (GOSSIP_IS_SESSION (session), NULL);
+
+	priv = GET_PRIV (session);
+
+	return priv->contact_manager;
+}
+
+GossipChatroomManager *
+gossip_session_get_chatroom_manager (GossipSession *session)
+{
+	GossipSessionPriv *priv;
+
+	g_return_val_if_fail (GOSSIP_IS_SESSION (session), NULL);
+
+	priv = GET_PRIV (session);
+
+	return priv->chatroom_manager;
+}
+
+GossipLogManager *
+gossip_session_get_log_manager (GossipSession *session)
+{
+	GossipSessionPriv *priv;
+
+	g_return_val_if_fail (GOSSIP_IS_SESSION (session), NULL);
+
+	priv = GET_PRIV (session);
+
+	return priv->log_manager;
+}
+
 static void
 session_get_accounts_foreach_cb (GossipAccount  *account,
 				 GossipProtocol *protocol,
@@ -841,7 +924,7 @@ gossip_session_new_account (GossipSession     *session,
 	protocol = gossip_protocol_new_from_account_type (type);
 	g_return_val_if_fail (GOSSIP_IS_PROTOCOL (protocol), NULL);
 
-	account = gossip_protocol_new_account (protocol);
+	account = gossip_protocol_new_account (protocol, type);
 	priv->protocols = g_list_append (priv->protocols,
 					 g_object_ref (protocol));
 
@@ -851,6 +934,8 @@ gossip_session_new_account (GossipSession     *session,
 
 	session_protocol_signals_setup (session, protocol);
 
+	gossip_protocol_setup (protocol, account);
+
 	g_object_unref (protocol);
 	
 	return account;
@@ -888,6 +973,8 @@ gossip_session_add_account (GossipSession *session,
 
 	session_protocol_signals_setup (session, protocol);
 
+	gossip_protocol_setup (protocol, account);
+
 	g_object_unref (protocol);
 
 	return TRUE;
@@ -899,6 +986,7 @@ gossip_session_remove_account (GossipSession *session,
 {
 	GossipSessionPriv *priv;
 	GossipProtocol    *protocol;
+	GList             *link;
 
 	g_return_val_if_fail (GOSSIP_IS_SESSION (session), FALSE);
 	g_return_val_if_fail (GOSSIP_IS_ACCOUNT (account), FALSE);
@@ -912,8 +1000,11 @@ gossip_session_remove_account (GossipSession *session,
 		return TRUE;
 	}
 
-	priv->protocols = g_list_remove (priv->protocols, protocol);
-	g_object_unref (protocol);
+	link = g_list_find (priv->protocols, protocol);
+	if (link) {
+		priv->protocols = g_list_delete_link (priv->protocols, link);
+		g_object_unref (protocol);
+	}
 
 	return g_hash_table_remove (priv->accounts, account);
 }
@@ -1023,12 +1114,6 @@ session_connect (GossipSession *session,
 		return;
 	}
 
-	g_signal_emit (session, signals[PROTOCOL_CONNECTING], 0, account, protocol);
-	priv->connecting_counter++;
-
-	/* Can we not just pass the GossipAccount on the GObject init? */
-	gossip_protocol_setup (protocol, account);
-
 	/* Setup the network connection */
 	gossip_protocol_login (protocol);
 }
@@ -1140,8 +1225,8 @@ session_disconnect (GossipSession *session,
 	g_return_if_fail (GOSSIP_IS_ACCOUNT (account));
 
 	protocol = g_hash_table_lookup (priv->accounts, account);
-
-	g_signal_emit (session, signals[PROTOCOL_DISCONNECTING], 0, account, protocol);
+	gossip_debug (DEBUG_DOMAIN, "about to disconnect account:%p, name:'%s'",
+		      account, gossip_account_get_name (account));
 
 	gossip_protocol_logout (protocol);
 }
@@ -1260,9 +1345,14 @@ gossip_session_set_presence (GossipSession  *session,
 	GList            *l;
 
 	g_return_if_fail (GOSSIP_IS_SESSION (session));
+	g_return_if_fail (GOSSIP_IS_PRESENCE (presence));
 
 	priv = GET_PRIV (session);
-	priv->presence = presence;
+
+	if (priv->presence) {
+		g_object_unref (priv->presence);
+	}
+	priv->presence = g_object_ref (presence);
 
 	for (l = priv->protocols; l; l = l->next) {
 		GossipProtocol *protocol;
@@ -1275,8 +1365,6 @@ gossip_session_set_presence (GossipSession  *session,
 
 		gossip_protocol_set_presence (protocol, presence);
 	}
-
-	g_signal_emit (session, signals[PRESENCE_CHANGED], 0, presence);
 }
 
 gboolean
@@ -1521,12 +1609,11 @@ gossip_session_rename_group (GossipSession *session,
 
 	priv = GET_PRIV (session);
 
-	/* 	protocol = session_get_protocol (session, NULL); */
-
 	/* FIXME: don't just blindly do this across all protocols
-	   actually pass the protocol in some how from the contact
-	   list? - what if we have the same group name in 2 different
-	   accounts */
+	 * actually pass the protocol in some how from the contact
+	 * list? - what if we have the same group name in 2 different
+	 * accounts.
+	 */
 	for (l = priv->protocols; l; l = l->next) {
 		GossipProtocol *protocol = GOSSIP_PROTOCOL (l->data);
 
@@ -1647,11 +1734,11 @@ gossip_session_get_nickname (GossipSession *session,
 }
 
 void
-gossip_session_register_account (GossipSession           *session,
-				 GossipAccount           *account,
-				 GossipVCard             *vcard,
-				 GossipRegisterCallback   callback,
-				 gpointer                 user_data)
+gossip_session_register_account (GossipSession       *session,
+				 GossipAccount       *account,
+				 GossipVCard         *vcard,
+				 GossipErrorCallback  callback,
+				 gpointer             user_data)
 {
 	GossipSessionPriv *priv;
 	GossipProtocol    *protocol;
@@ -1689,6 +1776,45 @@ gossip_session_register_cancel (GossipSession *session,
 	gossip_protocol_register_cancel (protocol);
 }
 
+void
+gossip_session_change_password (GossipSession       *session, 
+				GossipAccount       *account,
+				const gchar         *new_password,
+				GossipErrorCallback  callback,
+				gpointer             user_data)
+{
+	GossipSessionPriv *priv;
+	GossipProtocol    *protocol;
+
+	g_return_if_fail (GOSSIP_IS_SESSION (session));
+	g_return_if_fail (GOSSIP_IS_ACCOUNT (account));
+	g_return_if_fail (new_password != NULL);
+	g_return_if_fail (callback != NULL);
+
+	priv = GET_PRIV (session);
+
+	protocol = g_hash_table_lookup (priv->accounts, account);
+
+	gossip_protocol_change_password (protocol, new_password, callback, user_data);
+}
+
+void
+gossip_session_change_password_cancel (GossipSession *session,
+				       GossipAccount *account)
+{
+	GossipSessionPriv *priv;
+	GossipProtocol    *protocol;
+
+	g_return_if_fail (GOSSIP_IS_SESSION (session));
+	g_return_if_fail (GOSSIP_IS_ACCOUNT (account));
+
+	priv = GET_PRIV (session);
+
+	protocol = g_hash_table_lookup (priv->accounts, account);
+
+	gossip_protocol_change_password_cancel (protocol);
+}
+
 gboolean
 gossip_session_get_vcard (GossipSession        *session,
 			  GossipAccount        *account,
@@ -1742,12 +1868,12 @@ gossip_session_get_vcard (GossipSession        *session,
 }
 
 gboolean
-gossip_session_set_vcard (GossipSession         *session,
-			  GossipAccount         *account,
-			  GossipVCard           *vcard,
-			  GossipResultCallback   callback,
-			  gpointer               user_data,
-			  GError               **error)
+gossip_session_set_vcard (GossipSession   *session,
+			  GossipAccount   *account,
+			  GossipVCard     *vcard,
+			  GossipCallback   callback,
+			  gpointer         user_data,
+			  GError         **error)
 {
 	GossipSessionPriv *priv;
 	GList             *l;
@@ -1840,3 +1966,42 @@ gossip_session_get_version (GossipSession          *session,
 					    callback, user_data,
 					    error);
 }
+
+void
+gossip_session_chatroom_join_favorites (GossipSession *session)
+{
+	GossipSessionPriv *priv;
+	GList             *chatrooms;
+	GList             *l;
+
+	g_return_if_fail (GOSSIP_IS_SESSION (session));
+
+	priv = GET_PRIV (session);
+
+	chatrooms = gossip_chatroom_manager_get_chatrooms (priv->chatroom_manager, NULL);
+
+	for (l = chatrooms; l; l = l->next) {
+		GossipChatroomProvider *provider;
+		GossipChatroom         *chatroom;
+		GossipAccount          *account;
+
+		chatroom = l->data;
+
+		if (!gossip_chatroom_get_is_favourite (chatroom)) {
+			continue;
+		}
+
+		account = gossip_chatroom_get_account (chatroom);
+
+		if (!gossip_session_is_connected (session, account)) {
+			continue;
+		}
+
+		provider = gossip_session_get_chatroom_provider (session, account);
+
+		g_signal_emit (session, signals[CHATROOM_AUTO_CONNECT], 0,
+			       provider, chatroom);
+	}
+
+	g_list_free (chatrooms);
+}
