@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2005 Imendio AB
+ * Copyright (C) 2005-2007 Imendio AB
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -16,22 +16,31 @@
  * License along with this program; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
+ * 
+ * Authors: Martyn Russell <martyn@imendio.com>
  */
 
-#include <config.h>
+#include "config.h"
 
 #include <string.h>
 #include <stdlib.h>
 
 #include <loudmouth/loudmouth.h>
+
+#ifdef HAVE_GNOME
 #include <libgnomevfs/gnome-vfs.h>
+#endif
+
+#include <libgossip/gossip-contact.h>
+#include <libgossip/gossip-debug.h>
+#include <libgossip/gossip-presence.h>
+#include <libgossip/gossip-ft.h>
 
 #include "gossip-jabber-ft.h"
 #include "gossip-jabber-private.h"
 #include "gossip-jid.h"
 
-#define DEBUG_MSG(x) 
-/* #define DEBUG_MSG(args) g_printerr args ; g_printerr ("\n"); */
+#define DEBUG_DOMAIN "JabberFT"
 
 #define XMPP_FILE_TRANSFER_XMLNS        "http://jabber.org/protocol/si"
 #define XMPP_FILE_TRANSFER_PROFILE      "http://jabber.org/protocol/si/profile/file-transfer"
@@ -46,17 +55,15 @@
 
 #define IMPP_ERROR_XMLNS                "urn:ietf:params:xml:ns:xmpp-stanzas"
 
-
 struct _GossipJabberFTs {
 	GossipJabber   *jabber;
 	LmConnection   *connection;
 
- 	GHashTable     *str_ids; 
+	GHashTable     *str_ids;
 	GHashTable     *ft_ids;
 	GHashTable     *remote_ids;
 };
 
-
 static LmHandlerResult jabber_ft_iq_si_handler        (LmMessageHandler  *handler,
 						       LmConnection      *conn,
 						       LmMessage         *message,
@@ -76,13 +83,12 @@ static gboolean        jabber_ft_get_file_details     (const gchar       *uri,
 						       gchar            **mime_type);
 static gchar *         jabber_ft_get_contact_last_jid (GossipContact     *contact);
 
-
 GossipJabberFTs *
 gossip_jabber_ft_init (GossipJabber *jabber)
 {
 	GossipJabberFTs  *fts;
 	LmConnection     *connection;
- 	LmMessageHandler *handler;
+	LmMessageHandler *handler;
 
 	g_return_val_if_fail (GOSSIP_IS_JABBER (jabber), NULL);
 
@@ -107,7 +113,7 @@ gossip_jabber_ft_init (GossipJabber *jabber)
 						 g_direct_equal,
 						 NULL,
 						 g_free);
-	
+
 	handler = lm_message_handler_new ((LmHandleMessageFunction) jabber_ft_iq_si_handler,
 					  jabber, NULL);
 	lm_connection_register_message_handler (connection,
@@ -129,13 +135,13 @@ gossip_jabber_ft_finalize (GossipJabberFTs *fts)
 		return;
 	}
 
- 	g_hash_table_destroy (fts->str_ids); 
- 	g_hash_table_destroy (fts->ft_ids); 
- 	g_hash_table_destroy (fts->remote_ids); 
+	g_hash_table_destroy (fts->str_ids);
+	g_hash_table_destroy (fts->ft_ids);
+	g_hash_table_destroy (fts->remote_ids);
 
 	lm_connection_unref (fts->connection);
 	g_object_unref (fts->jabber);
-	
+
 	g_free (fts);
 }
 
@@ -145,13 +151,10 @@ jabber_ft_iq_si_handler (LmMessageHandler *handler,
 			 LmMessage        *m,
 			 GossipJabber     *jabber)
 {
-	GossipJabberPriv *priv;
 	LmMessageSubType  subtype;
 	LmMessageNode    *node;
 	const gchar      *xmlns;
 
-	priv = jabber->priv;
-	
 	subtype = lm_message_get_sub_type (m);
 
 	if (subtype != LM_MESSAGE_SUB_TYPE_GET &&
@@ -199,7 +202,7 @@ jabber_ft_iq_si_handler (LmMessageHandler *handler,
  * receiving
  */
 static void
-jabber_ft_handle_request (GossipJabber *jabber, 
+jabber_ft_handle_request (GossipJabber *jabber,
 			  LmMessage    *m)
 {
 	GossipJabberFTs *fts;
@@ -207,7 +210,7 @@ jabber_ft_handle_request (GossipJabber *jabber,
 	const gchar     *from_str;
 	GossipContact   *from;
 	GossipFT        *ft;
-	GossipFTId       id;             
+	GossipFTId       id;
 	LmMessageNode   *node;
 	const gchar     *file_name;
 	const gchar     *file_size;
@@ -219,43 +222,44 @@ jabber_ft_handle_request (GossipJabber *jabber,
 	id_str = lm_message_node_get_attribute (m->node, "id");
 
 	from_str = lm_message_node_get_attribute (m->node, "from");
-	from = gossip_jabber_get_contact_from_jid (jabber, from_str, NULL, TRUE);	
+	from = gossip_jabber_get_contact_from_jid (jabber, from_str, NULL, TRUE);
 
 	node = lm_message_node_find_child (m->node, "si");
-	file_mime = lm_message_node_get_attribute (node, "mime-type"); 
-	
-	node = lm_message_node_find_child (m->node, "file");
-	file_name = lm_message_node_get_attribute (node, "name"); 
-	file_size = lm_message_node_get_attribute (node, "size"); 
+	file_mime = lm_message_node_get_attribute (node, "mime-type");
 
-	ft = g_object_new (GOSSIP_TYPE_FT, 
+	node = lm_message_node_find_child (m->node, "file");
+	file_name = lm_message_node_get_attribute (node, "name");
+	file_size = lm_message_node_get_attribute (node, "size");
+
+	ft = g_object_new (GOSSIP_TYPE_FT,
 			   "type", GOSSIP_FT_TYPE_RECEIVING,
 			   "contact", from,
-			   "file-name", file_name, 
+			   "file-name", file_name,
 			   "file-size", g_ascii_strtoull (file_size, NULL, 10),
 			   "file-mime-type", file_mime,
 			   NULL);
-	
+
 	g_object_get (ft, "id", &id, NULL);
 
-	DEBUG_MSG (("ProtocolFT: ID[%d] File transfer request from:'%s' with file:'%s', size:'%s'", 
-		  id, 
-		  from_str,
-		  file_name,
-		  file_size));
+	gossip_debug (DEBUG_DOMAIN,
+		      "ID[%d] File transfer request from:'%s' with file:'%s', size:'%s'",
+		      id,
+		      from_str,
+		      file_name,
+		      file_size);
 
 	g_hash_table_insert (fts->str_ids, GUINT_TO_POINTER (id), g_strdup (id_str));
 	g_hash_table_insert (fts->ft_ids, g_strdup (id_str), ft);
 	g_hash_table_insert (fts->remote_ids, GUINT_TO_POINTER (id), g_strdup (from_str));
 
 	/* signal */
-	g_signal_emit_by_name (fts->jabber, 
-			       "file-transfer-request", 
+	g_signal_emit_by_name (fts->jabber,
+			       "file-transfer-request",
 			       ft);
 }
 
 static void
-jabber_ft_handle_error (GossipJabber *jabber, 
+jabber_ft_handle_error (GossipJabber *jabber,
 			LmMessage    *m)
 {
 	GossipJabberFTs *fts;
@@ -273,25 +277,26 @@ jabber_ft_handle_error (GossipJabber *jabber,
 
 	id_str = lm_message_node_get_attribute (m->node, "id");
 
+	ft = g_hash_table_lookup (fts->ft_ids, id_str);
+	if (!ft) {
+		gossip_debug (DEBUG_DOMAIN,
+			      "Could not find GossipFT* id:'%s'",
+			      id_str);
+		return;
+	}
+
 	from_str = lm_message_node_get_attribute (m->node, "from");
-	from = gossip_jabber_get_contact_from_jid (jabber, from_str, NULL, TRUE);	
+	from = gossip_jabber_get_contact_from_jid (jabber, from_str, NULL, TRUE);
 
 	node = lm_message_node_find_child (m->node, "error");
 	error_reason = lm_message_node_get_value (node);
-	error_code_str = lm_message_node_get_attribute (node, "code"); 
+	error_code_str = lm_message_node_get_attribute (node, "code");
 
-	DEBUG_MSG (("ProtocolFT: File transfer error from:'%s' with code:'%s', reason:'%s'", 
-		  from_str,
-		  error_code_str,
-		  error_reason));
-	
-
-	ft = g_hash_table_lookup (fts->ft_ids, id_str);
-	if (!ft) {
-		g_warning ("Could not find GossipFT* from id:'%s'", 
-			   id_str);
-		return;
-	}
+	gossip_debug (DEBUG_DOMAIN,
+		      "File transfer error from:'%s' with code:'%s', reason:'%s'",
+		      from_str,
+		      error_code_str,
+		      error_reason);
 
 	switch (atoi (error_code_str)) {
 	case 400:
@@ -306,8 +311,8 @@ jabber_ft_handle_error (GossipJabber *jabber,
 	}
 
 	/* signal */
-	jabber_ft_error (fts->jabber, 
-			 "file-transfer-error", 
+	jabber_ft_error (fts->jabber,
+			 "file-transfer-error",
 			 ft,
 			 error_code,
 			 error_reason ? error_reason : "");
@@ -320,21 +325,16 @@ jabber_ft_error (GossipJabber  *jabber,
 		 GossipFTError  code,
 		 const gchar   *reason)
 {
-	GError        *error;
-	static GQuark  quark = 0;
+	GError *error;
 
 	g_return_if_fail (GOSSIP_IS_JABBER (jabber));
 	g_return_if_fail (GOSSIP_IS_FT (ft));
 	g_return_if_fail (signal != NULL);
 	g_return_if_fail (reason != NULL);
 
-	if (!quark) {
-		quark = g_quark_from_static_string ("gossip-jabber-ft");
-	}
-
-	error = g_error_new_literal (quark, code, reason);
+	error = g_error_new_literal (gossip_jabber_error_quark(), code, reason);
 	g_signal_emit_by_name (jabber, signal, ft, error);
- 	g_error_free (error); 
+	g_error_free (error);
 }
 
 /*
@@ -346,35 +346,29 @@ jabber_ft_get_file_details (const gchar  *uri,
 			    gchar       **file_size,
 			    gchar       **mime_type)
 {
-	GnomeVFSFileInfo file_info;
-	GnomeVFSResult   result;
-	/*  	GnomeVFSURI      *uri;  */
+#ifdef HAVE_GNOME
+	GnomeVFSFileInfo *file_info;
+	GnomeVFSResult    result;
 
-	DEBUG_MSG (("ProtocolFT: Getting file info for URI:'%s'", uri));
+	gossip_debug (DEBUG_DOMAIN, "Getting file info for URI:'%s'", uri);
 
-	/*  	uri = gnome_vfs_uri_new (file_name); */
-
-	/* 	if (!gnome_vfs_uri_exists (uri)) { */
-	/* 		DEBUG_MSG (("ProtocolFT: URI:'%s' does not exist", file_name)); */
-	/* 		gnome_vfs_uri_unref (uri); */
-	/* 		return; */
-	/* 	} */
+	file_info = gnome_vfs_file_info_new ();
 
 	result = gnome_vfs_get_file_info (uri,
-					  &file_info, 
+					  file_info,
 					  GNOME_VFS_FILE_INFO_DEFAULT);
 
 	if (result != GNOME_VFS_OK) {
 		g_warning ("ProtocolFT: Could not get file info for URI:'%s'", uri);
 		return FALSE;
 	}
-	
+
 	if (file_name) {
-		*file_name = g_strdup (file_info.name);
+		*file_name = g_strdup (file_info->name);
 	}
 
 	if (file_size) {
-		*file_size = g_strdup_printf ("%d", (guint)file_info.size);
+		*file_size = g_strdup_printf ("%d", (guint) file_info->size);
 	}
 
 	if (mime_type) {
@@ -385,12 +379,14 @@ jabber_ft_get_file_details (const gchar  *uri,
 
 		if (!s) {
 			*mime_type = g_strdup (GNOME_VFS_MIME_TYPE_UNKNOWN);
-		} 
+		}
 	}
 
-	gnome_vfs_file_info_unref (&file_info);
-	
+	gnome_vfs_file_info_unref (file_info);
+
 	return TRUE;
+#endif
+	return FALSE;
 }
 
 static gchar *
@@ -408,7 +404,7 @@ jabber_ft_get_contact_last_jid (GossipContact *contact)
 	presence = gossip_contact_get_active_presence (contact);
 	resource = gossip_presence_get_resource (presence);
 
-	to = g_strdup_printf ("%s/%s", 
+	to = g_strdup_printf ("%s/%s",
 			      gossip_contact_get_id (contact),
 			      resource ? resource : "");
 
@@ -419,7 +415,7 @@ jabber_ft_get_contact_last_jid (GossipContact *contact)
  * sending
  */
 
-GossipFTId 
+GossipFTId
 gossip_jabber_ft_send (GossipJabberFTs *fts,
 		       GossipContact   *contact,
 		       const gchar     *file)
@@ -446,8 +442,8 @@ gossip_jabber_ft_send (GossipJabberFTs *fts,
 
 	to = jabber_ft_get_contact_last_jid (contact);
 
-	ok = jabber_ft_get_file_details (file, 
-					 &file_name, 
+	ok = jabber_ft_get_file_details (file,
+					 &file_name,
 					 &file_size,
 					 &mime_type);
 	if (!ok) {
@@ -461,13 +457,13 @@ gossip_jabber_ft_send (GossipJabberFTs *fts,
 
 	id_str = lm_message_node_get_attribute (m->node, "id");
 
-	lm_message_node_set_attribute (node, "from", 
+	lm_message_node_set_attribute (node, "from",
 				       gossip_contact_get_id (own_contact));
 
 	node = lm_message_node_add_child (node, "si", NULL);
-	lm_message_node_set_attributes (node, 
+	lm_message_node_set_attributes (node,
 					"profile", XMPP_FILE_TRANSFER_PROFILE,
- 					"mime-type", mime_type, 
+					"mime-type", mime_type,
 					"id", "ft1",
 					"xmlns", XMPP_FILE_TRANSFER_XMLNS,
 					NULL);
@@ -475,25 +471,25 @@ gossip_jabber_ft_send (GossipJabberFTs *fts,
 	si_node = node;
 
 	node = lm_message_node_add_child (si_node, "file", NULL);
-	lm_message_node_set_attributes (node, 
+	lm_message_node_set_attributes (node,
 					"size", file_size,
 					"name", file_name,
 					"xmlns", XMPP_FILE_XMLNS,
 					NULL);
 
 	node = lm_message_node_add_child (si_node, "feature", NULL);
-	lm_message_node_set_attributes (node, 
+	lm_message_node_set_attributes (node,
 					"xmlns", XMPP_FEATURE_XMLNS,
 					NULL);
 
 	node = lm_message_node_add_child (node, "x", NULL);
-	lm_message_node_set_attributes (node, 
+	lm_message_node_set_attributes (node,
 					"type", "form",
 					"xmlns", "jabber:x:data",
 					NULL);
 
 	node = lm_message_node_add_child (node, "field", NULL);
-	lm_message_node_set_attributes (node, 
+	lm_message_node_set_attributes (node,
 					"type", "list-single",
 					"var", "stream-method",
 					NULL);
@@ -504,29 +500,30 @@ gossip_jabber_ft_send (GossipJabberFTs *fts,
 /*  	node = lm_message_node_add_child (field_node, "option", NULL);  */
 /*  	lm_message_node_add_child (node, "value", XMPP_BYTESTREAMS_PROTOCOL);  */
 
- 	node = lm_message_node_add_child (field_node, "option", NULL);
- 	lm_message_node_add_child (node, "value", XMPP_IBB_PROTOCOL); 
+	node = lm_message_node_add_child (field_node, "option", NULL);
+	lm_message_node_add_child (node, "value", XMPP_IBB_PROTOCOL);
 
 	/* create object */
-	ft = g_object_new (GOSSIP_TYPE_FT, 
+	ft = g_object_new (GOSSIP_TYPE_FT,
 			   "type", GOSSIP_FT_TYPE_SENDING,
 			   "contact", contact,
-			   "file-name", file_name, 
+			   "file-name", file_name,
 			   "file-size", g_ascii_strtoull (file_size, NULL, 10),
 			   "file-mime-type", mime_type,
 			   NULL);
-	
+
 	g_object_get (ft, "id", &id, NULL);
 
-	DEBUG_MSG (("ProtocolFT: ID[%d] Sending file transfer request for URI:'%s'", 
-		  id, file));
+	gossip_debug (DEBUG_DOMAIN,
+		      "ID[%d] Sending file transfer request for URI:'%s'",
+		      id, file);
 
 	g_hash_table_insert (fts->str_ids, GUINT_TO_POINTER (id), g_strdup (id_str));
 	g_hash_table_insert (fts->ft_ids, g_strdup (id_str), ft);
 	g_hash_table_insert (fts->remote_ids, GUINT_TO_POINTER (id), to);
 
 	/* send */
- 	lm_connection_send (connection, m, NULL); 
+	lm_connection_send (connection, m, NULL);
 	lm_message_unref (m);
 
 	/* clean up */
@@ -537,9 +534,9 @@ gossip_jabber_ft_send (GossipJabberFTs *fts,
 	return id;
 }
 
-void 
+void
 gossip_jabber_ft_accept (GossipJabberFTs *fts,
-			 GossipFTId       id) 
+			 GossipFTId       id)
 {
 	GossipContact *own_contact;
 	const gchar   *id_str;
@@ -547,10 +544,10 @@ gossip_jabber_ft_accept (GossipJabberFTs *fts,
 	LmConnection  *connection;
 	LmMessage     *m;
 	LmMessageNode *node;
-	
+
 	g_return_if_fail (fts != NULL);
 
-	DEBUG_MSG (("ProtocolFT: ID[%d] Accepting file transfer", id));
+	gossip_debug (DEBUG_DOMAIN, "ID[%d] Accepting file transfer", id);
 
 	connection = gossip_jabber_get_connection (fts->jabber);
 	own_contact = gossip_jabber_get_own_contact (fts->jabber);
@@ -572,29 +569,29 @@ gossip_jabber_ft_accept (GossipJabberFTs *fts,
 					  LM_MESSAGE_TYPE_IQ,
 					  LM_MESSAGE_SUB_TYPE_RESULT);
 	node = m->node;
-	lm_message_node_set_attributes (node, 
+	lm_message_node_set_attributes (node,
 					"id", id_str,
 					"from", gossip_contact_get_id (own_contact),
 					NULL);
 
 	node = lm_message_node_add_child (node, "si", NULL);
-	lm_message_node_set_attributes (node, 
+	lm_message_node_set_attributes (node,
 					"xmlns", XMPP_FILE_TRANSFER_XMLNS,
 					NULL);
 
 	node = lm_message_node_add_child (node, "feature", NULL);
-	lm_message_node_set_attributes (node, 
+	lm_message_node_set_attributes (node,
 					"xmlns", XMPP_FEATURE_XMLNS,
 					NULL);
 
 	node = lm_message_node_add_child (node, "x", NULL);
-	lm_message_node_set_attributes (node, 
+	lm_message_node_set_attributes (node,
 					"xmlns", "jabber:x:data",
 					"type", "submit",
 					NULL);
 
 	node = lm_message_node_add_child (node, "field", NULL);
-	lm_message_node_set_attributes (node, 
+	lm_message_node_set_attributes (node,
 					"var", "stream-method",
 					NULL);
 
@@ -602,13 +599,13 @@ gossip_jabber_ft_accept (GossipJabberFTs *fts,
 	lm_message_node_add_child (node, "value", XMPP_IBB_PROTOCOL);
 
 	/* send */
- 	lm_connection_send (connection, m, NULL); 
+	lm_connection_send (connection, m, NULL);
 	lm_message_unref (m);
 }
 
-void 
+void
 gossip_jabber_ft_decline (GossipJabberFTs *fts,
-			  GossipFTId       id) 
+			  GossipFTId       id)
 {
 	GossipContact *own_contact;
 	const gchar   *id_str;
@@ -619,7 +616,7 @@ gossip_jabber_ft_decline (GossipJabberFTs *fts,
 
 	g_return_if_fail (fts != NULL);
 
-	DEBUG_MSG (("ProtocolFT: ID[%d] Declining file transfer", id));
+	gossip_debug (DEBUG_DOMAIN, "ID[%d] Declining file transfer", id);
 
 	connection = gossip_jabber_get_connection (fts->jabber);
 	own_contact = gossip_jabber_get_own_contact (fts->jabber);
@@ -642,19 +639,19 @@ gossip_jabber_ft_decline (GossipJabberFTs *fts,
 					  LM_MESSAGE_SUB_TYPE_ERROR);
 	node = m->node;
 
-	lm_message_node_set_attributes (node, 
-					"id", id_str,	
+	lm_message_node_set_attributes (node,
+					"id", id_str,
 					"from", gossip_contact_get_id (own_contact),
 					NULL);
 
 	node = lm_message_node_add_child (node, "error", "Declined");
-	lm_message_node_set_attributes (node, 
+	lm_message_node_set_attributes (node,
 					"code", "403",
 					NULL);
 
 	/* send */
- 	lm_connection_send (connection, m, NULL); 
-	lm_message_unref (m);	
+	lm_connection_send (connection, m, NULL);
+	lm_message_unref (m);
 
 	/* clean up hash tables */
 	g_hash_table_remove (fts->str_ids, &id);
@@ -671,7 +668,7 @@ gossip_jabber_ft_cancel (GossipJabberFTs *fts,
 /* 	room = (JabberChatroom*) g_hash_table_lookup (chatrooms->room_id_hash,  */
 /* 						       GINT_TO_POINTER (id)); */
 }
-		
+
 
 /*
  * stream stuff
@@ -698,14 +695,14 @@ gossip_jabber_ft_iib_start (GossipJabber *jabber,
 	node = m->node;
 
 	node = lm_message_node_add_child (node, "open", NULL);
-	lm_message_node_set_attributes (node, 
+	lm_message_node_set_attributes (node,
 					"sid", sid,
 					"block-size", block_size,
 					"xmlns", XMPP_IBB_PROTOCOL,
 					NULL);
 
 	/* send */
- 	lm_connection_send (connection, m, NULL); 
+	lm_connection_send (connection, m, NULL);
 	lm_message_unref (m);
 }
 
@@ -725,12 +722,12 @@ gossip_jabber_ft_iib_start_response (GossipJabber *jabber,
 	m = lm_message_new_with_sub_type (to,
 					  LM_MESSAGE_TYPE_IQ,
 					  LM_MESSAGE_SUB_TYPE_RESULT);
-	lm_message_node_set_attributes (m->node, 
+	lm_message_node_set_attributes (m->node,
 					"id", id,
 					NULL);
 
 	/* send */
- 	lm_connection_send (connection, m, NULL); 
+	lm_connection_send (connection, m, NULL);
 	lm_message_unref (m);
 }
 
@@ -759,23 +756,23 @@ gossip_jabber_ft_iib_error (GossipJabber *jabber,
 
 	node = m->node;
 
-	lm_message_node_set_attributes (node, 
+	lm_message_node_set_attributes (node,
 					"id", id,
 					NULL);
 
 	node = lm_message_node_add_child (node, "error", NULL);
-	lm_message_node_set_attributes (node, 
+	lm_message_node_set_attributes (node,
 					"code", error_code,
 					"type", error_type,
 					NULL);
 
 	node = lm_message_node_add_child (node, "feature-not-implemented", NULL);
-	lm_message_node_set_attributes (node, 
+	lm_message_node_set_attributes (node,
 					"xmlns", IMPP_ERROR_XMLNS,
 					NULL);
 
 	/* send */
- 	lm_connection_send (connection, m, NULL); 
+	lm_connection_send (connection, m, NULL);
 	lm_message_unref (m);
 }
 
@@ -802,35 +799,35 @@ gossip_jabber_ft_iib_send (GossipJabber *jabber,
 	node = m->node;
 
 	node = lm_message_node_add_child (node, "data", data);
-	lm_message_node_set_attributes (node, 
+	lm_message_node_set_attributes (node,
 					"xmlns", XMPP_IBB_PROTOCOL,
-					"sid", sid, 
+					"sid", sid,
 					"seq", seq,
 					NULL);
 
 	node = lm_message_node_add_child (node, "amp", NULL);
-	lm_message_node_set_attributes (node, 
+	lm_message_node_set_attributes (node,
 					"xmlns", XMPP_AMP_XMLNS,
 					NULL);
-	
+
 	amp_node = node;
 
 	node = lm_message_node_add_child (amp_node, "rule", NULL);
-	lm_message_node_set_attributes (node, 
+	lm_message_node_set_attributes (node,
 					"condition", "deliver-at",
 					"value", "stored",
 					"action", "error",
 					NULL);
 
 	node = lm_message_node_add_child (amp_node, "rule", NULL);
-	lm_message_node_set_attributes (node, 
+	lm_message_node_set_attributes (node,
 					"condition", "match-resource",
 					"value", "exact",
 					"action", "error",
 					NULL);
 
 	/* send */
- 	lm_connection_send (connection, m, NULL); 
+	lm_connection_send (connection, m, NULL);
 	lm_message_unref (m);
 }
 
@@ -848,20 +845,20 @@ gossip_jabber_ft_iib_finish (GossipJabber *jabber,
 
 	connection = gossip_jabber_get_connection (jabber);
 
-	m = lm_message_new_with_sub_type (to, 
+	m = lm_message_new_with_sub_type (to,
 					  LM_MESSAGE_TYPE_IQ,
 					  LM_MESSAGE_SUB_TYPE_SET);
 
 	node = m->node;
 
 	node = lm_message_node_add_child (node, "close", NULL);
-	lm_message_node_set_attributes (node, 
+	lm_message_node_set_attributes (node,
 					"xmlns", XMPP_IBB_PROTOCOL,
-					"sid", sid, 
+					"sid", sid,
 					NULL);
 
 	/* send */
- 	lm_connection_send (connection, m, NULL); 
+	lm_connection_send (connection, m, NULL);
 	lm_message_unref (m);
 }
 
@@ -881,12 +878,12 @@ gossip_jabber_ft_iib_finish_response (GossipJabber *jabber,
 	m = lm_message_new_with_sub_type (to,
 					  LM_MESSAGE_TYPE_IQ,
 					  LM_MESSAGE_SUB_TYPE_RESULT);
-	lm_message_node_set_attributes (m->node, 
+	lm_message_node_set_attributes (m->node,
 					"id", id,
 					NULL);
 
 	/* send */
- 	lm_connection_send (connection, m, NULL); 
+	lm_connection_send (connection, m, NULL);
 	lm_message_unref (m);
 }
 
