@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2005 Martyn Russell <martyn@imendio.com>
+ * Copyright (C) 2005-2007 Imendio AB
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -16,18 +16,18 @@
  * License along with this program; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
+ * 
+ * Authors: Martyn Russell <martyn@imendio.com>
  */
 
 #ifndef __GOSSIP_ACCOUNT_WIDGET_JABBER_H__
 #define __GOSSIP_ACCOUNT_WIDGET_JABBER_H__
 
-#include <gtk/gtkwidget.h>
 #include <libgossip/gossip-account.h>
 
 G_BEGIN_DECLS
 
-GtkWidget *gossip_account_widget_jabber_new (GossipAccount *account,
-					     GtkWidget     *label_name);
+GtkWidget *gossip_account_widget_jabber_new (GossipAccount *account);
 
 G_END_DECLS
 
