@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2005 Martyn Russell <mr@gnome.org>
+ * Copyright (C) 2005-2007 Imendio AB
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -16,35 +16,50 @@
  * License along with this program; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
+ * 
+ * Authors: Martyn Russell <martyn@imendio.com>
  */
 
-#include <config.h>
+#include "config.h"
+
 #include <string.h>
+
 #include <gtk/gtk.h>
 #include <glade/glade.h>
 #include <glib/gi18n.h>
 
-#include <libgossip/gossip-account.h>
+#include <libgossip/gossip-debug.h>
 #include <libgossip/gossip-protocol.h>
+#include <libgossip/gossip-session.h>
 #include <libgossip/gossip-utils.h>
+#include <libgossip/gossip-account-manager.h>
 
+#ifdef HAVE_TELEPATHY
+#include <protocols/telepathy/gossip-telepathy-cmgr.h>
+#include "gossip-protocol-chooser.h"
+#endif
+
+#include "gossip-app.h"
+#include "gossip-account-widget-irc.h"
 #include "gossip-account-widget-jabber.h"
+#include "gossip-account-widget-msn.h"
 #include "gossip-account-widget-generic.h"
 #include "gossip-accounts-dialog.h"
-#include "gossip-app.h"
+#include "gossip-glade.h"
+#include "gossip-ui-utils.h"
+
+#define DEBUG_DOMAIN "AccountsDialog"
 
 /* Flashing delay for icons (milliseconds). */
 #define FLASH_TIMEOUT 500
 
-#define STRING_EMPTY(x) ((x) == NULL || (x)[0] == '\0')
-
 typedef struct {
 	GtkWidget *window;
 
 	GtkWidget *alignment_settings;
 
 	GtkWidget *vbox_details;
-	GtkWidget *hbox_no_account;
+	GtkWidget *frame_no_account;
 	GtkWidget *label_no_account;
 	GtkWidget *label_no_account_blurb;
 
@@ -52,10 +67,16 @@ typedef struct {
 
 	GtkWidget *button_remove;
 	GtkWidget *button_connect;
-	GtkWidget *button_register;
 
-	GtkWidget *label_name;
+	GtkWidget *frame_new_account;
+	GtkWidget *combobox_type;
 	GtkWidget *entry_name;
+	GtkWidget *button_create;
+	GtkWidget *button_back;
+
+	GtkWidget *image_type;
+	GtkWidget *label_name;
+	GtkWidget *label_type;
 	GtkWidget *checkbutton_connect;
 	GtkWidget *checkbutton_proxy;
 	GtkWidget *settings_widget;
@@ -71,94 +92,94 @@ typedef struct {
 	GtkComboBox   *combobox;
 } SetAccountData;
 
-static void           accounts_dialog_setup                      (GossipAccountsDialog  *dialog);
-static void           accounts_dialog_update_connect_button      (GossipAccountsDialog  *dialog);
-static void           accounts_dialog_update_account             (GossipAccountsDialog  *dialog,
-								  GossipAccount         *account);   
-static void           accounts_dialog_model_setup                (GossipAccountsDialog  *dialog);
-static void           accounts_dialog_model_add_columns          (GossipAccountsDialog  *dialog);
-static void           accounts_dialog_model_pixbuf_data_func     (GtkTreeViewColumn     *tree_column,
-								  GtkCellRenderer       *cell,
-								  GtkTreeModel          *model,
-								  GtkTreeIter           *iter,
-								  GossipAccountsDialog  *dialog);
-static GossipAccount *accounts_dialog_model_get_selected         (GossipAccountsDialog  *dialog);
-static void           accounts_dialog_model_set_selected         (GossipAccountsDialog  *dialog,
-								  GossipAccount         *account);
-static void           accounts_dialog_model_select_first         (GossipAccountsDialog  *dialog);
-static gboolean       accounts_dialog_model_remove_selected      (GossipAccountsDialog  *dialog);
-static void           accounts_dialog_model_selection_changed    (GtkTreeSelection      *selection,
-								  GossipAccountsDialog  *dialog);
-static void           accounts_dialog_model_cell_edited          (GtkCellRendererText   *cell,
-								  const gchar           *path_string,
-								  const gchar           *new_text,
-								  GossipAccountsDialog  *dialog);
-static void           accounts_dialog_account_added_cb           (GossipAccountManager  *manager,
-								  GossipAccount         *account,
-								  GossipAccountsDialog  *dialog);
-static void           accounts_dialog_account_removed_cb         (GossipAccountManager  *manager,
-								  GossipAccount         *account,
-								  GossipAccountsDialog  *dialog);
-static void           accounts_dialog_account_name_changed_cb    (GossipAccount         *account,
-								  GParamSpec            *param,
-								  GossipAccountsDialog  *dialog);
-static gboolean       accounts_dialog_flash_connecting_foreach   (GtkTreeModel          *model,
-								  GtkTreePath           *path,
-								  GtkTreeIter           *iter,
-								  gpointer               user_data);
-static gboolean       accounts_dialog_flash_connecting_cb        (GossipAccountsDialog  *dialog);
-static void           accounts_dialog_protocol_connecting_cb     (GossipSession         *session,
-								  GossipAccount         *account,
-								  GossipProtocol        *protocol,
-								  GossipAccountsDialog  *dialog);
-static void           accounts_dialog_protocol_connected_cb      (GossipSession         *session,
-								  GossipAccount         *account,
-								  GossipProtocol        *protocol,
-								  GossipAccountsDialog  *dialog);
-static void           accounts_dialog_protocol_disconnected_cb   (GossipSession         *session,
-								  GossipAccount         *account,
-								  GossipProtocol        *protocol,
-								  gint                   reason,
-								  GossipAccountsDialog  *dialog);
-static void           accounts_dialog_protocol_error_cb          (GossipSession         *session,
-								  GossipProtocol        *protocol,
-								  GossipAccount         *account,
-								  GError                *error,
-								  GossipAccountsDialog  *dialog);
-static void           accounts_dialog_register_cancel            (GossipAccountsDialog  *dialog);
-static void           accounts_dialog_register_cb                (GossipResult           result,
-								  GError                *error,
-								  GossipAccountsDialog  *dialog);
-static void           accounts_dialog_button_register_clicked_cb (GtkWidget             *button,
-								  GossipAccountsDialog  *dialog);
-static void           accounts_dialog_button_add_clicked_cb      (GtkWidget             *button,
-								  GossipAccountsDialog  *dialog);
-static void           accounts_dialog_remove_response_cb         (GtkWidget             *dialog,
-								  gint                   response,
-								  GossipAccount         *account);
-static void           accounts_dialog_button_remove_clicked_cb   (GtkWidget             *button,
-								 GossipAccountsDialog   *dialog);
-static void           accounts_dialog_treeview_row_activated_cb  (GtkTreeView           *tree_view,
-								  GtkTreePath           *path,
-								  GtkTreeViewColumn     *column,
-								  gpointer              *dialog);
-static void           accounts_dialog_save                       (GossipAccountsDialog  *dialog);
-static void           accounts_dialog_entry_changed_cb           (GtkWidget             *widget,
-								  GossipAccountsDialog  *dialog);
-static gboolean       accounts_dialog_entry_focus_cb             (GtkWidget             *widget,
-								  GdkEventFocus         *event,
-								  GossipAccountsDialog  *dialog);
-static void           accounts_dialog_checkbutton_toggled_cb     (GtkWidget             *widget,
-								  GossipAccountsDialog  *dialog);
-static gboolean       accounts_dialog_foreach                    (GtkTreeModel          *model,
-								  GtkTreePath           *path,
-								  GtkTreeIter           *iter,
-								  GossipAccountsDialog  *dialog);
-static void           accounts_dialog_response_cb                (GtkWidget             *widget,
-								  gint                   response,
-								  GossipAccountsDialog  *dialog);
-static void           accounts_dialog_destroy_cb                 (GtkWidget             *widget,
-								  GossipAccountsDialog  *dialog);
+static void           accounts_dialog_setup                     (GossipAccountsDialog *dialog);
+static void           accounts_dialog_set_connecting            (GossipAccountsDialog *dialog,
+								 GossipAccount        *account);
+static void           accounts_dialog_update_connect_button     (GossipAccountsDialog *dialog);
+static void           accounts_dialog_update_account            (GossipAccountsDialog *dialog,
+								 GossipAccount        *account);
+static void           accounts_dialog_model_setup               (GossipAccountsDialog *dialog);
+static void           accounts_dialog_model_add_columns         (GossipAccountsDialog *dialog);
+static void           accounts_dialog_model_pixbuf_data_func    (GtkTreeViewColumn    *tree_column,
+								 GtkCellRenderer      *cell,
+								 GtkTreeModel         *model,
+								 GtkTreeIter          *iter,
+								 GossipAccountsDialog *dialog);
+static GossipAccount *accounts_dialog_model_get_selected        (GossipAccountsDialog *dialog);
+static void           accounts_dialog_model_set_selected        (GossipAccountsDialog *dialog,
+								 GossipAccount        *account);
+static void           accounts_dialog_model_select_first        (GossipAccountsDialog *dialog);
+static gboolean       accounts_dialog_model_remove_selected     (GossipAccountsDialog *dialog);
+static void           accounts_dialog_model_selection_changed   (GtkTreeSelection     *selection,
+								 GossipAccountsDialog *dialog);
+static void           accounts_dialog_model_cell_edited         (GtkCellRendererText  *cell,
+								 const gchar          *path_string,
+								 const gchar          *new_text,
+								 GossipAccountsDialog *dialog);
+static void           accounts_dialog_account_added_cb          (GossipAccountManager *manager,
+								 GossipAccount        *account,
+								 GossipAccountsDialog *dialog);
+static void           accounts_dialog_account_removed_cb        (GossipAccountManager *manager,
+								 GossipAccount        *account,
+								 GossipAccountsDialog *dialog);
+static void           accounts_dialog_account_name_changed_cb   (GossipAccount        *account,
+								 GParamSpec           *param,
+								 GossipAccountsDialog *dialog);
+static gboolean       accounts_dialog_flash_connecting_foreach  (GtkTreeModel         *model,
+								 GtkTreePath          *path,
+								 GtkTreeIter          *iter,
+								 gpointer              user_data);
+static gboolean       accounts_dialog_flash_connecting_cb       (GossipAccountsDialog *dialog);
+static void           accounts_dialog_protocol_connecting_cb    (GossipSession        *session,
+								 GossipAccount        *account,
+								 GossipProtocol       *protocol,
+								 GossipAccountsDialog *dialog);
+static void           accounts_dialog_protocol_connected_cb     (GossipSession        *session,
+								 GossipAccount        *account,
+								 GossipProtocol       *protocol,
+								 GossipAccountsDialog *dialog);
+static void           accounts_dialog_protocol_disconnected_cb  (GossipSession        *session,
+								 GossipAccount        *account,
+								 GossipProtocol       *protocol,
+								 gint                  reason,
+								 GossipAccountsDialog *dialog);
+static void           accounts_dialog_protocol_error_cb         (GossipSession        *session,
+								 GossipProtocol       *protocol,
+								 GossipAccount        *account,
+								 GError               *error,
+								 GossipAccountsDialog *dialog);
+static void           accounts_dialog_entry_name_changed_cb     (GtkWidget            *widget,
+								 GossipAccountsDialog *dialog);
+static void           accounts_dialog_button_create_clicked_cb  (GtkWidget            *button,
+								 GossipAccountsDialog *dialog);
+static void           accounts_dialog_button_back_clicked_cb    (GtkWidget            *button,
+								 GossipAccountsDialog *dialog);
+static void           accounts_dialog_button_connect_clicked_cb (GtkWidget            *button,
+								 GossipAccountsDialog *dialog);
+static void           accounts_dialog_button_add_clicked_cb     (GtkWidget            *button,
+								 GossipAccountsDialog *dialog);
+static void           accounts_dialog_remove_response_cb        (GtkWidget            *dialog,
+								 gint                  response,
+								 GossipAccount        *account);
+static void           accounts_dialog_button_remove_clicked_cb  (GtkWidget            *button,
+								 GossipAccountsDialog *dialog);
+static void           accounts_dialog_treeview_row_activated_cb (GtkTreeView          *tree_view,
+								 GtkTreePath          *path,
+								 GtkTreeViewColumn    *column,
+								 gpointer             *dialog);
+static void           accounts_dialog_save                      (GossipAccountsDialog *dialog);
+static void           accounts_dialog_checkbutton_toggled_cb    (GtkWidget            *widget,
+								 GossipAccountsDialog *dialog);
+static gboolean       accounts_dialog_foreach                   (GtkTreeModel         *model,
+								 GtkTreePath          *path,
+								 GtkTreeIter          *iter,
+								 GossipAccountsDialog *dialog);
+static void           accounts_dialog_response_cb               (GtkWidget            *widget,
+								 gint                  response,
+								 GossipAccountsDialog *dialog);
+static void           accounts_dialog_destroy_cb                (GtkWidget            *widget,
+								 GossipAccountsDialog *dialog);
+
 enum {
 	COL_NAME,
 	COL_EDITABLE,
@@ -196,8 +217,9 @@ accounts_dialog_setup (GossipAccountsDialog *dialog)
 	for (l = accounts; l; l = l->next) {
 		GossipAccount *account;
 		const gchar   *name;
-		gboolean       is_default = FALSE;
-		gboolean       is_connected = FALSE;
+		gboolean       is_default;
+		gboolean       is_connected;
+		gboolean       is_connecting;
 
 		account = l->data;
 
@@ -206,27 +228,101 @@ accounts_dialog_setup (GossipAccountsDialog *dialog)
 			continue;
 		}
 
-		is_default = (gossip_account_equal (account, default_account));
-		is_connected = 	(gossip_session_is_connected (session, account));
+		is_default = gossip_account_equal (account, default_account);
+		is_connected = gossip_session_is_connected (session, account);
+		is_connecting =	gossip_session_is_connecting (session, account);
 
-		gtk_list_store_append (store, &iter);
-		gtk_list_store_set (store, &iter,
-				    COL_NAME, name,
-				    COL_EDITABLE, is_editable,
-				    COL_DEFAULT, is_default,
-				    COL_CONNECTED, is_connected,
-				    COL_AUTO_CONNECT, gossip_account_get_auto_connect (account),
-				    COL_ACCOUNT_POINTER, account,
-				    -1);
+		gossip_debug (DEBUG_DOMAIN,
+			      "Adding account:'%s', connected:%s, connecting:%s",
+			      name, 
+			      is_connected ? "YES" : "NO",
+			      is_connecting ? "YES" : "NO");
+
+		gtk_list_store_insert_with_values (store, &iter,
+						   -1,
+						   COL_NAME, name,
+						   COL_EDITABLE, is_editable,
+						   COL_DEFAULT, is_default,
+						   COL_CONNECTED, is_connected,
+						   COL_CONNECTING, is_connecting,
+						   COL_AUTO_CONNECT, gossip_account_get_auto_connect (account),
+						   COL_ACCOUNT_POINTER, account,
+						   -1);
 
 		g_signal_connect (account, "notify::name",
-				  G_CALLBACK (accounts_dialog_account_name_changed_cb), dialog);
+				  G_CALLBACK (accounts_dialog_account_name_changed_cb), 
+				  dialog);
+
+		if (is_connecting) {
+			accounts_dialog_set_connecting (dialog, account);
+		}
 	}
 
 	g_list_foreach (accounts, (GFunc) g_object_unref, NULL);
 	g_list_free (accounts);
 }
 
+static void
+accounts_dialog_set_connecting (GossipAccountsDialog *dialog,
+				GossipAccount        *account)
+{
+	GtkTreeView      *view;
+	GtkTreeSelection *selection;
+	GtkTreeModel     *model;
+	GtkTreeIter       iter;
+	gboolean          ok;
+	GossipAccount    *selected_account;
+
+	if (!dialog->connecting_id) {
+		dialog->connecting_id = g_timeout_add (FLASH_TIMEOUT,
+						       (GSourceFunc) accounts_dialog_flash_connecting_cb,
+						       dialog);
+	}
+
+	selected_account = accounts_dialog_model_get_selected (dialog);
+	if (selected_account) {
+		if (gossip_account_equal (selected_account, account)) {
+			gtk_widget_set_sensitive (dialog->button_remove, FALSE);
+		}
+
+		g_object_unref (selected_account);
+	}
+
+	view = GTK_TREE_VIEW (dialog->treeview);
+	selection = gtk_tree_view_get_selection (view);
+	model = gtk_tree_view_get_model (view);
+
+	for (ok = gtk_tree_model_get_iter_first (model, &iter);
+	     ok;
+	     ok = gtk_tree_model_iter_next (model, &iter)) {
+		GossipAccount *this_account;
+		gboolean       equal;
+
+		gtk_tree_model_get (model, &iter,
+				    COL_ACCOUNT_POINTER, &this_account,
+				    -1);
+
+		equal = gossip_account_equal (this_account, account);
+		g_object_unref (this_account);
+
+		if (equal) {
+			GtkTreePath *path;
+
+			gtk_list_store_set (GTK_LIST_STORE (model), &iter,
+					    COL_CONNECTING, TRUE,
+					    -1);
+
+			path = gtk_tree_model_get_path (model, &iter);
+			gtk_tree_model_row_changed (model, path, &iter);
+			gtk_tree_path_free (path);
+
+			break;
+		}
+	}
+
+	accounts_dialog_update_connect_button (dialog);
+}
+
 static void
 accounts_dialog_update_connect_button (GossipAccountsDialog *dialog)
 {
@@ -270,8 +366,6 @@ accounts_dialog_update_connect_button (GossipAccountsDialog *dialog)
 
 	gtk_button_set_label (GTK_BUTTON (dialog->button_connect), label);
 	gtk_button_set_image (GTK_BUTTON (dialog->button_connect), image);
-
-	gtk_widget_set_sensitive (dialog->button_register, !is_connected && !is_connecting);
 }
 
 static void
@@ -287,11 +381,10 @@ accounts_dialog_update_account (GossipAccountsDialog *dialog,
 		GtkTreeView  *view;
 		GtkTreeModel *model;
 
-		gtk_widget_show (dialog->hbox_no_account);
+		gtk_widget_show (dialog->frame_no_account);
 		gtk_widget_hide (dialog->vbox_details);
 
 		gtk_widget_set_sensitive (dialog->button_connect, FALSE);
-		gtk_widget_set_sensitive (dialog->button_register, FALSE);
 		gtk_widget_set_sensitive (dialog->button_remove, FALSE);
 
 		view = GTK_TREE_VIEW (dialog->treeview);
@@ -317,21 +410,32 @@ accounts_dialog_update_account (GossipAccountsDialog *dialog,
 						"for you to start configuring."));
 		}
 	} else {
-		gtk_widget_hide (dialog->hbox_no_account);
+		gtk_widget_hide (dialog->frame_no_account);
 		gtk_widget_show (dialog->vbox_details);
 
-		/* FIXME: Should have some way to know if we _CAN_ register */
-		gtk_widget_set_sensitive (dialog->button_register, TRUE);
-
 		switch (gossip_account_get_type (account)) {
-		case GOSSIP_ACCOUNT_TYPE_JABBER:
-			dialog->settings_widget = gossip_account_widget_jabber_new 
-				(account, dialog->label_name);
+		case GOSSIP_ACCOUNT_TYPE_IRC:
+			dialog->settings_widget = 
+				gossip_account_widget_irc_new (account);
 			break;
+
+		case GOSSIP_ACCOUNT_TYPE_JABBER_LEGACY:
+		case GOSSIP_ACCOUNT_TYPE_JABBER:
+			dialog->settings_widget = 
+				gossip_account_widget_jabber_new (account);
+			break;
+
+		case GOSSIP_ACCOUNT_TYPE_MSN:
+			dialog->settings_widget = 
+				gossip_account_widget_msn_new (account);
+			break;
+
 		default:
-			dialog->settings_widget = gossip_account_widget_generic_new 
-				(account, dialog->label_name);
+			dialog->settings_widget = 
+				gossip_account_widget_generic_new (account);
 		}
+		
+		gtk_widget_grab_focus (dialog->settings_widget);
 	}
 
 	if (dialog->settings_widget) {
@@ -340,7 +444,21 @@ accounts_dialog_update_account (GossipAccountsDialog *dialog,
 	}
 
 	if (account) {
-		gtk_entry_set_text (GTK_ENTRY (dialog->entry_name), 
+		GossipAccountType  account_type;
+		const gchar       *account_type_str;
+		GdkPixbuf         *pixbuf;
+
+		pixbuf = gossip_account_create_pixbuf (account,
+						       GTK_ICON_SIZE_DIALOG);
+		gtk_image_set_from_pixbuf (GTK_IMAGE (dialog->image_type), pixbuf);
+		if (pixbuf) {
+			g_object_unref (pixbuf);
+		}
+
+		account_type = gossip_account_get_type (account);
+		account_type_str = gossip_account_type_to_string (account_type);
+		gtk_label_set_text (GTK_LABEL (dialog->label_type), account_type_str);
+		gtk_label_set_text (GTK_LABEL (dialog->label_name), 
 				    gossip_account_get_name (account));
 
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog->checkbutton_proxy),
@@ -458,7 +576,7 @@ accounts_dialog_model_pixbuf_data_func (GtkTreeViewColumn    *tree_column,
 			    COL_ACCOUNT_POINTER, &account,
 			    -1);
 
-	pixbuf = gossip_pixbuf_from_account (account, GTK_ICON_SIZE_BUTTON);
+	pixbuf = gossip_account_create_pixbuf (account, GTK_ICON_SIZE_BUTTON);
 
 	if (pixbuf) {
 		if ((!is_connecting && !is_connected) ||
@@ -742,10 +860,7 @@ accounts_dialog_flash_connecting_foreach (GtkTreeModel *model,
 	gboolean is_connecting;
 
 	gtk_tree_model_get (model, iter, COL_CONNECTING, &is_connecting, -1);
-
-/* 	if (is_connecting) { */
-		gtk_tree_model_row_changed (model, path, iter);
-/* 	} */
+	gtk_tree_model_row_changed (model, path, iter);
 
 	return FALSE;
 }
@@ -772,61 +887,7 @@ accounts_dialog_protocol_connecting_cb (GossipSession        *session,
 					GossipProtocol       *protocol,
 					GossipAccountsDialog *dialog)
 {
-	GtkTreeView      *view;
-	GtkTreeSelection *selection;
-	GtkTreeModel     *model;
-	GtkTreeIter       iter;
-	gboolean          ok;
-	GossipAccount    *selected_account;
-
-	if (!dialog->connecting_id) {
-		dialog->connecting_id = g_timeout_add (FLASH_TIMEOUT,
-						       (GSourceFunc) accounts_dialog_flash_connecting_cb,
-						       dialog);
-	}
-
-	selected_account = accounts_dialog_model_get_selected (dialog);
-	if (selected_account) {
-		if (gossip_account_equal (selected_account, account)) {
-			gtk_widget_set_sensitive (dialog->button_remove, FALSE);
-		}
-
-		g_object_unref (selected_account);
-	}
-
-	view = GTK_TREE_VIEW (dialog->treeview);
-	selection = gtk_tree_view_get_selection (view);
-	model = gtk_tree_view_get_model (view);
-
-	for (ok = gtk_tree_model_get_iter_first (model, &iter);
-	     ok;
-	     ok = gtk_tree_model_iter_next (model, &iter)) {
-		GossipAccount *this_account;
-		gboolean       equal;
-
-		gtk_tree_model_get (model, &iter,
-				    COL_ACCOUNT_POINTER, &this_account,
-				    -1);
-
-		equal = gossip_account_equal (this_account, account);
-		g_object_unref (this_account);
-
-		if (equal) {
-			GtkTreePath *path;
-
-			gtk_list_store_set (GTK_LIST_STORE (model), &iter,
-					    COL_CONNECTING, TRUE,
-					    -1);
-
-			path = gtk_tree_model_get_path (model, &iter);
-			gtk_tree_model_row_changed (model, path, &iter);
-			gtk_tree_path_free (path);
-
-			break;
-		}
-	}
-
-	accounts_dialog_update_connect_button (dialog);
+	accounts_dialog_set_connecting (dialog, account);
 }
 
 static void
@@ -1026,6 +1087,99 @@ accounts_dialog_protocol_error_cb (GossipSession        *session,
 	accounts_dialog_update_connect_button (dialog);
 }
 
+static void          
+accounts_dialog_entry_name_changed_cb (GtkWidget             *widget,
+				       GossipAccountsDialog  *dialog)
+{
+	const gchar *str;
+	
+	str = gtk_entry_get_text (GTK_ENTRY (widget));
+	gtk_widget_set_sensitive (dialog->button_create, !G_STR_EMPTY (str));
+}
+
+static void
+accounts_dialog_button_create_clicked_cb (GtkWidget             *button,
+					  GossipAccountsDialog  *dialog)
+{
+	GossipSession        *session;
+	GossipAccountManager *manager;
+	GossipAccount        *account;
+	GossipAccountType     account_type;
+	gchar                *account_type_str;
+	const gchar          *str;
+
+	/* Update widgets */
+	gtk_widget_show (dialog->vbox_details);
+	gtk_widget_hide (dialog->frame_no_account);
+	gtk_widget_hide (dialog->frame_new_account);
+
+	/* Get account type */
+	account_type_str = gtk_combo_box_get_active_text (GTK_COMBO_BOX (dialog->combobox_type));
+	if (!account_type_str) {
+		return;
+	}
+
+	account_type = GOSSIP_ACCOUNT_TYPE_UNKNOWN;
+
+	str = gossip_account_type_to_string (GOSSIP_ACCOUNT_TYPE_JABBER_LEGACY);
+	if (strcmp (account_type_str, str) == 0) {
+		account_type = GOSSIP_ACCOUNT_TYPE_JABBER_LEGACY;
+	} 
+
+#ifdef HAVE_TELEPATHY
+ 	str = gossip_account_type_to_string (GOSSIP_ACCOUNT_TYPE_JABBER);
+	if (strcmp (account_type_str, str) == 0) {
+		account_type = GOSSIP_ACCOUNT_TYPE_JABBER;
+	} 
+
+ 	str = gossip_account_type_to_string (GOSSIP_ACCOUNT_TYPE_MSN);
+	if (strcmp (account_type_str, str) == 0) {
+		account_type = GOSSIP_ACCOUNT_TYPE_MSN;
+	}
+
+ 	str = gossip_account_type_to_string (GOSSIP_ACCOUNT_TYPE_IRC);
+	if (strcmp (account_type_str, str) == 0) {
+		account_type = GOSSIP_ACCOUNT_TYPE_IRC;
+	}
+
+ 	str = gossip_account_type_to_string (GOSSIP_ACCOUNT_TYPE_SALUT);
+	if (strcmp (account_type_str, str) == 0) {
+		account_type = GOSSIP_ACCOUNT_TYPE_SALUT;
+	}
+#endif
+	
+	if (account_type == GOSSIP_ACCOUNT_TYPE_UNKNOWN) {
+		g_warning ("We have an unknown account type selected in the GtkComboBox");
+		return;
+	}
+
+	/* Create account */
+	session = gossip_app_get_session ();
+	manager = gossip_session_get_account_manager (session);
+	account = gossip_session_new_account (session, account_type);
+
+	str = gtk_entry_get_text (GTK_ENTRY (dialog->entry_name));
+	gossip_account_set_name (account, str);
+
+	gossip_account_manager_add (manager, account);
+	accounts_dialog_model_set_selected (dialog, account);
+	g_object_unref (account);
+}
+
+static void
+accounts_dialog_button_back_clicked_cb (GtkWidget             *button,
+					GossipAccountsDialog  *dialog)
+{
+	GossipAccount *account;
+
+	gtk_widget_hide (dialog->vbox_details);
+	gtk_widget_hide (dialog->frame_no_account);
+	gtk_widget_hide (dialog->frame_new_account);
+
+	account = accounts_dialog_model_get_selected (dialog);
+	accounts_dialog_update_account (dialog, account);
+}
+
 static void
 accounts_dialog_button_connect_clicked_cb (GtkWidget            *button,
 					   GossipAccountsDialog *dialog)
@@ -1082,130 +1236,16 @@ accounts_dialog_button_connect_clicked_cb (GtkWidget            *button,
 	g_object_unref (account);
 }
 
-static void
-accounts_dialog_register_cancel (GossipAccountsDialog *dialog)
-{
-	GossipSession *session;
-	GossipAccount *account;
-
-	if (!dialog->registering) {
-		return;
-	}
-
-	session = gossip_app_get_session ();
-	account = accounts_dialog_model_get_selected (dialog);
-	gossip_session_register_cancel (session,
-					account);
-
-	dialog->registering = FALSE;
-	g_object_unref (account);
-}
-
-static void
-accounts_dialog_register_cb (GossipResult          result,
-			     GError               *error,
-			     GossipAccountsDialog *dialog)
-{
-	GtkWidget *md;
-
-	dialog->registering = FALSE;
-
-	/* FIXME: Not sure how to do this right, but really we
-	 * shouldn't show the register button as sensitive if we have
-	 * just registered.
-	 */
-	gtk_widget_set_sensitive (dialog->button_register, TRUE);
-
-	if (result == GOSSIP_RESULT_OK) {
-		md = gtk_message_dialog_new (GTK_WINDOW (dialog->window),
-					     GTK_DIALOG_MODAL,
-					     GTK_MESSAGE_INFO,
-					     GTK_BUTTONS_CLOSE,
-					     _("Successfully registered your new account settings."));
-
-		gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (md),
-							  _("You should now be able to connect to your new account."));
-
-	} else {
-		md = gtk_message_dialog_new (GTK_WINDOW (dialog->window),
-					     GTK_DIALOG_MODAL,
-					     GTK_MESSAGE_ERROR,
-					     GTK_BUTTONS_CLOSE,
-					     _("Failed to register your new account settings."));
-		
-		if (error && error->message) {
-			gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (md),
-								  error->message);
-		}
-	}
-
-	g_signal_connect_swapped (md,
-				  "response", 
-				  G_CALLBACK (gtk_widget_destroy),
-				  md);
-
-	gtk_widget_show_all (md);
-}
-
-static void
-accounts_dialog_button_register_clicked_cb (GtkWidget            *button,
-					    GossipAccountsDialog *dialog)
-{
-	GossipSession *session;
-	GossipAccount *account;
-	GossipVCard   *vcard;
-	gchar         *nickname;
-	const gchar   *name;
-	const gchar   *last_part;
-
-	dialog->registering = TRUE;
-	gtk_widget_set_sensitive (dialog->button_register, FALSE);
-
-	session = gossip_app_get_session ();
-	account = accounts_dialog_model_get_selected (dialog);
-	vcard = gossip_vcard_new ();
-	name = gossip_account_get_name (account);
-
-	last_part = strstr (name, " ");
-	if (last_part) {
-		gint len;
-
-		len = last_part - name;
-		nickname = g_strndup (name, len);
-	} else {
-		nickname = g_strdup (name);
-	}
-
-	gossip_vcard_set_name (vcard, name);
-	gossip_vcard_set_nickname (vcard, nickname);
-
-	g_free (nickname);
-
-	gossip_session_register_account (session,
-					 account,
-					 vcard,
-					 (GossipRegisterCallback) accounts_dialog_register_cb,
-					 dialog);
-
-	g_object_unref (account);
-}
-
 static void
 accounts_dialog_button_add_clicked_cb (GtkWidget            *button,
 				       GossipAccountsDialog *dialog)
 {
-	GossipAccountManager *manager;
-	GossipAccount        *account;
-	GossipSession        *session;
-
-	session = gossip_app_get_session ();
-	manager = gossip_session_get_account_manager (session);
-	account = gossip_session_new_account (session, GOSSIP_ACCOUNT_TYPE_JABBER);
-
-	gossip_account_manager_add (manager, account);
-	accounts_dialog_model_set_selected (dialog, account);
-	g_object_unref (account);
+	gtk_widget_hide (dialog->vbox_details);
+	gtk_widget_hide (dialog->frame_no_account);
+	gtk_widget_show (dialog->frame_new_account);
 
+	gtk_combo_box_set_active (GTK_COMBO_BOX (dialog->combobox_type), 0);
+	gtk_entry_set_text (GTK_ENTRY (dialog->entry_name), "");
 	gtk_widget_grab_focus (dialog->entry_name);
 }
 
@@ -1243,7 +1283,7 @@ accounts_dialog_button_remove_clicked_cb (GtkWidget            *button,
 		(GTK_WINDOW (dialog->window),
 		 GTK_DIALOG_DESTROY_WITH_PARENT,
 		 GTK_MESSAGE_QUESTION,
-		 GTK_BUTTONS_YES_NO,
+		 GTK_BUTTONS_NONE,
 		 _("You are about to remove your %s account!\n"
 		   "Are you sure you want to proceed?"),
 		 gossip_account_get_name (account));
@@ -1256,6 +1296,13 @@ accounts_dialog_button_remove_clicked_cb (GtkWidget            *button,
 		   "Should you decide to add the account back at a later time, "
 		   "they will still be available."));
 
+	gtk_dialog_add_button (GTK_DIALOG (message_dialog),
+			       GTK_STOCK_CANCEL, 
+			       GTK_RESPONSE_NO);
+	gtk_dialog_add_button (GTK_DIALOG (message_dialog),
+			       GTK_STOCK_REMOVE, 
+			       GTK_RESPONSE_YES);
+
 	g_signal_connect (message_dialog, "response",
 			  G_CALLBACK (accounts_dialog_remove_response_cb),
 			  account);
@@ -1281,7 +1328,6 @@ accounts_dialog_save (GossipAccountsDialog *dialog)
 	GossipSession        *session;
 	GossipAccount        *account;
 	GossipAccountManager *manager;
- 	const gchar          *str;
 	gboolean              bool;
 
 	dialog->account_changed = FALSE;
@@ -1294,9 +1340,6 @@ accounts_dialog_save (GossipAccountsDialog *dialog)
 		return;
 	}
 
-	str = gtk_entry_get_text (GTK_ENTRY (dialog->entry_name));
- 	gossip_account_set_name (account, str); 
-
 	bool = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->checkbutton_connect));
 	gossip_account_set_auto_connect (account, bool);
 
@@ -1308,40 +1351,6 @@ accounts_dialog_save (GossipAccountsDialog *dialog)
 	g_object_unref (account);
 }
 
-static void
-accounts_dialog_entry_changed_cb (GtkWidget            *widget,
-				  GossipAccountsDialog *dialog)
-{
-	dialog->account_changed = TRUE;
-}
-
-static gboolean
-accounts_dialog_entry_focus_cb (GtkWidget            *widget,
-				GdkEventFocus        *event,
-				GossipAccountsDialog *dialog)
-{
-	GossipAccount *account;
-	const gchar   *str;
-
- 	account = accounts_dialog_model_get_selected (dialog);
-
-	str = gtk_entry_get_text (GTK_ENTRY (widget));
-	if (STRING_EMPTY (str)) {
-		str = gossip_account_get_name (account);
-
-		gtk_entry_set_text (GTK_ENTRY (widget), str);
-		dialog->account_changed = FALSE;
-	}
-
-	if (dialog->account_changed) {
- 		accounts_dialog_save (dialog); 
-	}
-
-	g_object_unref (account);
-
-	return FALSE;
-}
-
 static void  
 accounts_dialog_checkbutton_toggled_cb (GtkWidget            *widget,
 					GossipAccountsDialog *dialog)
@@ -1384,8 +1393,6 @@ accounts_dialog_destroy_cb (GtkWidget            *widget,
 	GossipAccountManager *manager;
 	GtkTreeModel         *model;
 
-	accounts_dialog_register_cancel (dialog);
-
 	session = gossip_app_get_session ();
 	manager = gossip_session_get_account_manager (session);
 
@@ -1436,7 +1443,9 @@ gossip_accounts_dialog_show (GossipAccount *account)
 	GossipAccountManager        *manager;
 	static GossipAccountsDialog *dialog = NULL;
 	GladeXML                    *glade;
-	GtkWidget                   *bbox, *button_close;
+	GtkWidget                   *bbox;
+	GtkWidget                   *button_close;
+	const gchar                 *str;
 
 	if (dialog) {
 		gtk_window_present (GTK_WINDOW (dialog->window));
@@ -1450,41 +1459,65 @@ gossip_accounts_dialog_show (GossipAccount *account)
 				       NULL,
 				       "accounts_dialog", &dialog->window,
 				       "vbox_details", &dialog->vbox_details,
-				       "hbox_no_account", &dialog->hbox_no_account,
+				       "frame_no_account", &dialog->frame_no_account,
 				       "label_no_account", &dialog->label_no_account,
 				       "label_no_account_blurb", &dialog->label_no_account_blurb,
 				       "alignment_settings", &dialog->alignment_settings,
 				       "dialog-action_area", &bbox,
 				       "treeview", &dialog->treeview,
+				       "frame_new_account", &dialog->frame_new_account,
+				       "combobox_type", &dialog->combobox_type,
+				       "entry_name", &dialog->entry_name,
+				       "button_create", &dialog->button_create,
+				       "button_back", &dialog->button_back,
+				       "image_type", &dialog->image_type,
+				       "label_type", &dialog->label_type,
+				       "label_name", &dialog->label_name,
+				       "checkbutton_connect", &dialog->checkbutton_connect,
+				       "checkbutton_proxy", &dialog->checkbutton_proxy,
 				       "button_remove", &dialog->button_remove,
 				       "button_connect", &dialog->button_connect,
 				       "button_close", &button_close,
-				       "button_register", &dialog->button_register,
-				       "label_name", &dialog->label_name,
-				       "entry_name", &dialog->entry_name,
-				       "checkbutton_connect", &dialog->checkbutton_connect,
-				       "checkbutton_proxy", &dialog->checkbutton_proxy,
 				       NULL);
 
 	gossip_glade_connect (glade,
 			      dialog,
 			      "accounts_dialog", "destroy", accounts_dialog_destroy_cb,
 			      "accounts_dialog", "response", accounts_dialog_response_cb,
-			      "button_connect", "clicked", accounts_dialog_button_connect_clicked_cb,
-			      "button_register", "clicked", accounts_dialog_button_register_clicked_cb,
-			      "button_add", "clicked", accounts_dialog_button_add_clicked_cb,
-			      "button_remove", "clicked", accounts_dialog_button_remove_clicked_cb,
+			      "button_create", "clicked", accounts_dialog_button_create_clicked_cb,
+			      "button_back", "clicked", accounts_dialog_button_back_clicked_cb,
+			      "entry_name", "changed", accounts_dialog_entry_name_changed_cb,
 			      "treeview", "row-activated", accounts_dialog_treeview_row_activated_cb,
-			      "entry_name", "changed", accounts_dialog_entry_changed_cb,
-			      "entry_name", "focus-out-event", accounts_dialog_entry_focus_cb,
 			      "checkbutton_connect", "toggled", accounts_dialog_checkbutton_toggled_cb,
 			      "checkbutton_proxy", "toggled", accounts_dialog_checkbutton_toggled_cb,
+			      "button_connect", "clicked", accounts_dialog_button_connect_clicked_cb,
+			      "button_add", "clicked", accounts_dialog_button_add_clicked_cb,
+			      "button_remove", "clicked", accounts_dialog_button_remove_clicked_cb,
 			      NULL);
 
 	g_object_add_weak_pointer (G_OBJECT (dialog->window), (gpointer) &dialog);
 
 	g_object_unref (glade);
 
+	/* Set up combo */
+	str = gossip_account_type_to_string (GOSSIP_ACCOUNT_TYPE_JABBER_LEGACY);
+	gtk_combo_box_append_text (GTK_COMBO_BOX (dialog->combobox_type), str);
+
+#ifdef HAVE_TELEPATHY
+ 	str = gossip_account_type_to_string (GOSSIP_ACCOUNT_TYPE_JABBER);
+	gtk_combo_box_append_text (GTK_COMBO_BOX (dialog->combobox_type), str);
+
+ 	str = gossip_account_type_to_string (GOSSIP_ACCOUNT_TYPE_MSN);
+	gtk_combo_box_append_text (GTK_COMBO_BOX (dialog->combobox_type), str);
+
+ 	str = gossip_account_type_to_string (GOSSIP_ACCOUNT_TYPE_IRC);
+	gtk_combo_box_append_text (GTK_COMBO_BOX (dialog->combobox_type), str);
+
+ 	str = gossip_account_type_to_string (GOSSIP_ACCOUNT_TYPE_SALUT);
+	gtk_combo_box_append_text (GTK_COMBO_BOX (dialog->combobox_type), str);
+#endif
+
+	/* Set up signalling */
 	session = gossip_app_get_session ();
 	manager = gossip_session_get_account_manager (session);
 
@@ -1517,14 +1550,15 @@ gossip_accounts_dialog_show (GossipAccount *account)
 
 	gtk_window_set_transient_for (GTK_WINDOW (dialog->window),
 				      GTK_WINDOW (gossip_app_get_window ()));
-	gtk_widget_show (dialog->window);
 
 	if (GOSSIP_IS_ACCOUNT (account)) {
-		/* if account was specified then we select it */
+		/* If account was specified then we select it */
 		accounts_dialog_model_set_selected (dialog, account);
 	} else {
 		accounts_dialog_model_select_first (dialog);
 	}
+
+	gtk_widget_show (dialog->window);
 }
 
 gboolean
