@@ -16,8 +16,6 @@
  * License along with this program; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
- * 
- * Authors: Martyn Russell <martyn@imendio.com>
  */
 
 #include "config.h"
@@ -39,6 +37,7 @@
 #include "gossip-add-contact-dialog.h"
 #include "gossip-app.h"
 #include "gossip-avatar-image.h"
+#include "gossip-glade.h"
 #include "gossip-ui-utils.h"
 
 #define DEBUG_DOMAIN "AddContact"
@@ -157,7 +156,7 @@ add_contact_dialog_vcard_cb (GossipResult            result,
 		}
 		
 		/* Avatar */
-		pixbuf = gossip_pixbuf_avatar_from_vcard (vcard);
+		pixbuf = gossip_avatar_create_pixbuf_with_size (gossip_vcard_get_avatar (vcard), 48);
 		if (pixbuf != NULL) {
 			gossip_avatar_image_set_pixbuf (GOSSIP_AVATAR_IMAGE (dialog->avatar_image), 
 							pixbuf);
