@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2002-2004 Imendio AB
+ * Copyright (C) 2002-2007 Imendio AB
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -21,15 +21,18 @@
 #ifndef __GOSSIP_APP_H__
 #define __GOSSIP_APP_H__
 
-#include <gtk/gtkwidget.h>
+#include <glib-object.h>
 
-#include <libgossip/gossip-account-manager.h>
-#include <libgossip/gossip-chatroom-manager.h>
-#include <libgossip/gossip-event-manager.h>
+#include <libgossip/gossip-account.h>
 #include <libgossip/gossip-session.h>
+#include <libgossip/gossip-event-manager.h>
+#include <libgossip/gossip-chatroom-manager.h>
 
+#include "gossip-heartbeat.h"
+#include "gossip-self-presence.h"
 #include "gossip-chat-manager.h"
-#include "gossip-ui-utils.h"
+
+G_BEGIN_DECLS
 
 #define GOSSIP_TYPE_APP         (gossip_app_get_type ())
 #define GOSSIP_APP(o)           (G_TYPE_CHECK_INSTANCE_CAST ((o), GOSSIP_TYPE_APP, GossipApp))
@@ -43,11 +46,11 @@ typedef struct _GossipAppClass GossipAppClass;
 typedef struct _GossipAppPriv  GossipAppPriv;
 
 struct _GossipApp {
-        GObject parent;
+	GObject parent;
 };
 
 struct _GossipAppClass {
-        GObjectClass parent_class;
+	GObjectClass parent_class;
 };
 
 GType                  gossip_app_get_type             (void) G_GNUC_CONST;
@@ -55,8 +58,7 @@ void                   gossip_app_connect              (GossipAccount        *ac
 							gboolean              startup);
 void                   gossip_app_net_down             (void);
 void                   gossip_app_net_up               (void);
-void                   gossip_app_create               (GossipSession        *session,
-							GossipAccountManager *manager);
+void                   gossip_app_create               (GossipSession        *session);
 GossipApp *            gossip_app_get                  (void);
 gboolean               gossip_app_is_connected         (void);
 gboolean               gossip_app_is_window_visible    (void);
@@ -70,5 +72,9 @@ GossipSession *        gossip_app_get_session          (void);
 GossipChatroomManager *gossip_app_get_chatroom_manager (void);
 GossipChatManager *    gossip_app_get_chat_manager     (void);
 GossipEventManager *   gossip_app_get_event_manager    (void);
+GossipSelfPresence *   gossip_app_get_self_presence    (void);
+GossipHeartbeat *      gossip_app_get_flash_heartbeat  (void);
+
+G_END_DECLS
 
 #endif /* __GOSSIP_APP_H__ */
