@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2003-2006 Imendio AB
+ * Copyright (C) 2002-2007 Imendio AB
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -18,32 +18,37 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <config.h>
+#include "config.h"
+
 #include <sys/types.h>
 #include <string.h>
 #include <time.h>
+
 #include <glib/gi18n.h>
+#include <gtk/gtkbutton.h>
 #include <gtk/gtkimage.h>
 #include <gtk/gtkmenu.h>
 #include <gtk/gtkmenuitem.h>
 #include <gtk/gtkimagemenuitem.h>
 #include <gtk/gtkstock.h>
 #include <gtk/gtkscrolledwindow.h>
+#include <gtk/gtksizegroup.h>
+#include <glade/glade.h>
 
-#include <libgossip/gossip-debug.h>
-#include <libgossip/gossip-conf.h>
-#include <libgossip/gossip-time.h>
 #include <libgossip/gossip-utils.h>
+#include <libgossip/gossip-account.h>
+#include <libgossip/gossip-chatroom.h>
 #include <libgossip/gossip-session.h>
+#include <libgossip/gossip-debug.h>
 #include <libgossip/gossip-chatroom-provider.h>
+#include <libgossip/gossip-conf.h>
 
 #include "gossip-app.h"
-#include "gossip-chat-view.h"
 #include "gossip-preferences.h"
 #include "gossip-theme-manager.h"
-
-#define GET_PRIV(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), \
-		       GOSSIP_TYPE_CHAT_VIEW, GossipChatViewPriv))
+#include "gossip-text-iter.h"
+#include "gossip-ui-utils.h"
+#include "gossip-chat-view.h"
 
 #define DEBUG_DOMAIN "ChatView"
 
@@ -52,6 +57,8 @@
 
 #define MAX_LINES 800
 
+#define GET_PRIV(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GOSSIP_TYPE_CHAT_VIEW, GossipChatViewPriv))
+
 typedef enum {
 	BLOCK_TYPE_NONE,
 	BLOCK_TYPE_SELF,
@@ -71,8 +78,10 @@ struct _GossipChatViewPriv {
 	gboolean       allow_scrolling;
 	gboolean       is_group_chat;
 
-	GtkTextMark   *find_mark;
+	GtkTextMark   *find_mark_previous;
+	GtkTextMark   *find_mark_next;
 	gboolean       find_wrapped;
+	gboolean       find_last_direction;
 
 	/* This is for the group chat so we know if the "other" last contact
 	 * changed, so we know whether to insert a header or not.
@@ -266,7 +275,7 @@ gossip_chat_view_class_init (GossipChatViewClass *klass)
 
 	object_class->finalize = chat_view_finalize;
 	widget_class->size_allocate = chat_view_size_allocate;
-	widget_class->drag_motion = chat_view_drag_motion;
+ 	widget_class->drag_motion = chat_view_drag_motion; 
 
 	g_type_class_add_private (object_class, sizeof (GossipChatViewPriv));
 }
@@ -361,7 +370,7 @@ chat_view_drag_motion (GtkWidget        *widget,
 	 * the result of dragging something across it.
 	 */
 
-	return TRUE;
+	return FALSE;
 }
 
 static void
@@ -530,7 +539,7 @@ chat_view_populate_popup (GossipChatView *view,
 						&start, &end, FALSE);
 	}
 
-	if (!str || strlen (str) < 1) {
+	if (G_STR_EMPTY (str)) {
 		g_free (str);
 		return;
 	}
@@ -1421,7 +1430,12 @@ chat_view_invite_join_cb (GossipChatroomProvider   *provider,
 			  gint                      id,
 			  gpointer                  user_data)
 {
-	gossip_group_chat_new (provider, id);
+	GossipChatroom  *chatroom;
+	GossipGroupChat *chat;
+
+	chatroom = gossip_chatroom_provider_find_by_id (provider, id);
+	chat = gossip_group_chat_new (provider, chatroom);
+/* 	g_object_unref (chat); */
 }
 
 static void
@@ -1471,6 +1485,10 @@ gossip_chat_view_append_message_from_self (GossipChatView *view,
 	const gchar        *body;
 	gboolean            scroll_down;
 
+	g_return_if_fail (GOSSIP_IS_CHAT_VIEW (view));
+	g_return_if_fail (GOSSIP_IS_MESSAGE (msg));
+	g_return_if_fail (GOSSIP_IS_CONTACT (my_contact));
+
 	priv = GET_PRIV (view);
 
 	body = gossip_message_get_body (msg);
@@ -1530,6 +1548,10 @@ gossip_chat_view_append_message_from_other (GossipChatView *view,
 	const gchar        *body;
 	gboolean            scroll_down;
 
+	g_return_if_fail (GOSSIP_IS_CHAT_VIEW (view));
+	g_return_if_fail (GOSSIP_IS_MESSAGE (msg));
+	g_return_if_fail (GOSSIP_IS_CONTACT (contact));
+
 	priv = GET_PRIV (view);
 
 	body = gossip_message_get_body (msg);
@@ -1588,6 +1610,9 @@ gossip_chat_view_append_event (GossipChatView *view,
 	gchar              *msg;
 	const gchar        *tag;
 
+	g_return_if_fail (GOSSIP_IS_CHAT_VIEW (view));
+	g_return_if_fail (!G_STR_EMPTY (str));
+
 	priv = GET_PRIV (view);
 
 	bottom = chat_view_is_scrolled_down (view);
@@ -1642,6 +1667,8 @@ gossip_chat_view_append_invite (GossipChatView *view,
 	const gchar          *tag;
 	GString              *s;
 
+	g_return_if_fail (GOSSIP_IS_CHAT_VIEW (view));
+
 	priv = GET_PRIV (view);
 
 	if (priv->irc_style) {
@@ -1661,12 +1688,12 @@ gossip_chat_view_append_invite (GossipChatView *view,
 	reason = gossip_chatroom_invite_get_reason (invite);
 
 	s = g_string_new ("");
-	if (body && strlen (body) > 0) {
+	if (!G_STR_EMPTY (body)) {
 		s = g_string_append (s, body);
 	}
 
 	/* Make sure the reason is not the body (or in the body) */
-	if (reason && strlen (reason) > 0 && !strstr (body, reason)) {
+	if (!G_STR_EMPTY (reason) && !strstr (body, reason)) {
 		if (s->len > 0) {
 			s = g_string_append_c (s, '\n');
 		}
@@ -1759,12 +1786,88 @@ gossip_chat_view_append_invite (GossipChatView *view,
 	priv->last_block_type = BLOCK_TYPE_INVITE;
 }
 
+void
+gossip_chat_view_append_button (GossipChatView *view,
+				const gchar    *message,
+				GtkWidget      *button1,
+				GtkWidget      *button2)
+{
+	GossipChatViewPriv   *priv;
+	GtkTextChildAnchor   *anchor;
+	GtkTextIter           iter;
+	gboolean              bottom;
+	const gchar          *tag;
+
+	g_return_if_fail (GOSSIP_IS_CHAT_VIEW (view));
+	g_return_if_fail (button1 != NULL);
+
+	priv = GET_PRIV (view);
+
+	if (priv->irc_style) {
+		tag = "irc-invite";
+	} else {
+		tag = "fancy-invite";
+	}
+
+	bottom = chat_view_is_scrolled_down (view);
+
+	chat_view_maybe_append_date_and_time (view, NULL);
+
+	if (message) {
+		chat_view_append_text (view, message, tag);
+	}
+
+	gtk_text_buffer_get_end_iter (priv->buffer, &iter);
+
+	anchor = gtk_text_buffer_create_child_anchor (priv->buffer, &iter);
+	gtk_text_view_add_child_at_anchor (GTK_TEXT_VIEW (view), button1, anchor);
+	gtk_widget_show (button1);
+
+	gtk_text_buffer_insert_with_tags_by_name (priv->buffer,
+						  &iter,
+						  " ",
+						  1,
+						  tag,
+						  NULL);
+
+	if (button2) {
+		gtk_text_buffer_get_end_iter (priv->buffer, &iter);
+		
+		anchor = gtk_text_buffer_create_child_anchor (priv->buffer, &iter);
+		gtk_text_view_add_child_at_anchor (GTK_TEXT_VIEW (view), button2, anchor);
+		gtk_widget_show (button2);
+		
+		gtk_text_buffer_insert_with_tags_by_name (priv->buffer,
+							  &iter,
+							  " ",
+							  1,
+							  tag,
+							  NULL);
+	}
+
+	gtk_text_buffer_get_end_iter (priv->buffer, &iter);
+	gtk_text_buffer_insert_with_tags_by_name (priv->buffer,
+						  &iter,
+						  "\n\n",
+						  2,
+						  tag,
+						  NULL);
+
+	if (bottom) {
+		gossip_chat_view_scroll_down (view);
+	}
+
+	priv->last_block_type = BLOCK_TYPE_INVITE;
+}
+
 void
 gossip_chat_view_scroll (GossipChatView *view,
 			 gboolean        allow_scrolling)
 {
 	GossipChatViewPriv *priv;
 
+	g_return_if_fail (GOSSIP_IS_CHAT_VIEW (view));
+
 	priv = GET_PRIV (view);
 
 	priv->allow_scrolling = allow_scrolling;
@@ -1781,6 +1884,8 @@ gossip_chat_view_scroll_down (GossipChatView *view)
 	GtkTextIter         iter;
 	GtkTextMark        *mark;
 
+	g_return_if_fail (GOSSIP_IS_CHAT_VIEW (view));
+
 	priv = GET_PRIV (view);
 
 	if (!priv->allow_scrolling) {
@@ -1842,10 +1947,10 @@ gossip_chat_view_clear (GossipChatView *view)
 	priv->last_timestamp = 0;
 }
 
-void
-gossip_chat_view_find (GossipChatView *view,
-		       const gchar    *search_criteria,
-		       gboolean        new_search)
+gboolean
+gossip_chat_view_find_previous (GossipChatView *view,
+				const gchar    *search_criteria,
+				gboolean        new_search)
 {
 	GossipChatViewPriv *priv;
 	GtkTextBuffer      *buffer;
@@ -1855,22 +1960,22 @@ gossip_chat_view_find (GossipChatView *view,
 	gboolean            found;
 	gboolean            from_start = FALSE;
 
-	g_return_if_fail (GOSSIP_IS_CHAT_VIEW (view));
-	g_return_if_fail (search_criteria != NULL);
+	g_return_val_if_fail (GOSSIP_IS_CHAT_VIEW (view), FALSE);
+	g_return_val_if_fail (search_criteria != NULL, FALSE);
 
 	priv = GET_PRIV (view);
 
 	buffer = priv->buffer;
 
-	if (strlen (search_criteria) == 0) {
-		if (priv->find_mark) {
+	if (G_STR_EMPTY (search_criteria)) {
+		if (priv->find_mark_previous) {
 			gtk_text_buffer_get_start_iter (buffer, &iter_at_mark);
 
 			gtk_text_buffer_move_mark (buffer,
-						   priv->find_mark,
+						   priv->find_mark_previous,
 						   &iter_at_mark);
 			gtk_text_view_scroll_to_mark (GTK_TEXT_VIEW (view),
-						      priv->find_mark,
+						      priv->find_mark_previous,
 						      0.0,
 						      TRUE,
 						      0.0,
@@ -1880,22 +1985,139 @@ gossip_chat_view_find (GossipChatView *view,
 						      &iter_at_mark);
 		}
 
-		return;
+		return FALSE;
 	}
 
 	if (new_search) {
 		from_start = TRUE;
 	}
 
-	if (priv->find_mark) {
+	if (priv->find_mark_previous) {
 		gtk_text_buffer_get_iter_at_mark (buffer,
 						  &iter_at_mark,
-						  priv->find_mark);
+						  priv->find_mark_previous);
+	} else {
+		gtk_text_buffer_get_end_iter (buffer, &iter_at_mark);
+		from_start = TRUE;
+	}
+
+	priv->find_last_direction = FALSE;
+
+	found = gossip_text_iter_backward_search (&iter_at_mark,
+						  search_criteria,
+						  &iter_match_start,
+						  &iter_match_end,
+						  NULL);
+
+	if (!found) {
+		gboolean result = FALSE;
+
+		if (from_start) {
+			return result;
+		}
+
+		/* Here we wrap around. */
+		if (!new_search && !priv->find_wrapped) {
+			priv->find_wrapped = TRUE;
+			result = gossip_chat_view_find_previous (view, 
+								 search_criteria, 
+								 FALSE);
+			priv->find_wrapped = FALSE;
+		}
+
+		return result;
+	}
+
+	/* Set new mark and show on screen */
+	if (!priv->find_mark_previous) {
+		priv->find_mark_previous = gtk_text_buffer_create_mark (buffer, NULL,
+									&iter_match_start,
+									TRUE);
+	} else {
+		gtk_text_buffer_move_mark (buffer,
+					   priv->find_mark_previous,
+					   &iter_match_start);
+	}
+
+	if (!priv->find_mark_next) {
+		priv->find_mark_next = gtk_text_buffer_create_mark (buffer, NULL,
+								    &iter_match_end,
+								    TRUE);
+	} else {
+		gtk_text_buffer_move_mark (buffer,
+					   priv->find_mark_next,
+					   &iter_match_end);
+	}
+
+	gtk_text_view_scroll_to_mark (GTK_TEXT_VIEW (view),
+				      priv->find_mark_previous,
+				      0.0,
+				      TRUE,
+				      0.5,
+				      0.5);
+
+	gtk_text_buffer_move_mark_by_name (buffer, "selection_bound", &iter_match_start);
+	gtk_text_buffer_move_mark_by_name (buffer, "insert", &iter_match_end);
+
+	return TRUE;
+}
+
+gboolean
+gossip_chat_view_find_next (GossipChatView *view,
+			    const gchar    *search_criteria,
+			    gboolean        new_search)
+{
+	GossipChatViewPriv *priv;
+	GtkTextBuffer      *buffer;
+	GtkTextIter         iter_at_mark;
+	GtkTextIter         iter_match_start;
+	GtkTextIter         iter_match_end;
+	gboolean            found;
+	gboolean            from_start = FALSE;
+
+	g_return_val_if_fail (GOSSIP_IS_CHAT_VIEW (view), FALSE);
+	g_return_val_if_fail (search_criteria != NULL, FALSE);
+
+	priv = GET_PRIV (view);
+
+	buffer = priv->buffer;
+
+	if (G_STR_EMPTY (search_criteria)) {
+		if (priv->find_mark_next) {
+			gtk_text_buffer_get_start_iter (buffer, &iter_at_mark);
+
+			gtk_text_buffer_move_mark (buffer,
+						   priv->find_mark_next,
+						   &iter_at_mark);
+			gtk_text_view_scroll_to_mark (GTK_TEXT_VIEW (view),
+						      priv->find_mark_next,
+						      0.0,
+						      TRUE,
+						      0.0,
+						      0.0);
+			gtk_text_buffer_select_range (buffer,
+						      &iter_at_mark,
+						      &iter_at_mark);
+		}
+
+		return FALSE;
+	}
+
+	if (new_search) {
+		from_start = TRUE;
+	}
+
+	if (priv->find_mark_next) {
+		gtk_text_buffer_get_iter_at_mark (buffer,
+						  &iter_at_mark,
+						  priv->find_mark_next);
 	} else {
 		gtk_text_buffer_get_start_iter (buffer, &iter_at_mark);
 		from_start = TRUE;
 	}
 
+	priv->find_last_direction = TRUE;
+
 	found = gossip_text_iter_forward_search (&iter_at_mark,
 						 search_criteria,
 						 &iter_match_start,
@@ -1903,33 +2125,47 @@ gossip_chat_view_find (GossipChatView *view,
 						 NULL);
 
 	if (!found) {
+		gboolean result = FALSE;
+
 		if (from_start) {
-			return;
+			return result;
 		}
 
 		/* Here we wrap around. */
 		if (!new_search && !priv->find_wrapped) {
 			priv->find_wrapped = TRUE;
-			gossip_chat_view_find (view, search_criteria, FALSE);
+			result = gossip_chat_view_find_next (view, 
+							     search_criteria, 
+							     FALSE);
 			priv->find_wrapped = FALSE;
 		}
 
-		return;
+		return result;
 	}
 
 	/* Set new mark and show on screen */
-	if (!priv->find_mark) {
-		priv->find_mark = gtk_text_buffer_create_mark (buffer, NULL,
+	if (!priv->find_mark_next) {
+		priv->find_mark_next = gtk_text_buffer_create_mark (buffer, NULL,
 							       &iter_match_end,
 							       TRUE);
 	} else {
 		gtk_text_buffer_move_mark (buffer,
-					   priv->find_mark,
+					   priv->find_mark_next,
 					   &iter_match_end);
 	}
 
+	if (!priv->find_mark_previous) {
+		priv->find_mark_previous = gtk_text_buffer_create_mark (buffer, NULL,
+									&iter_match_start,
+									TRUE);
+	} else {
+		gtk_text_buffer_move_mark (buffer,
+					   priv->find_mark_previous,
+					   &iter_match_start);
+	}
+
 	gtk_text_view_scroll_to_mark (GTK_TEXT_VIEW (view),
-				      priv->find_mark,
+				      priv->find_mark_next,
 				      0.0,
 				      TRUE,
 				      0.5,
@@ -1937,6 +2173,62 @@ gossip_chat_view_find (GossipChatView *view,
 
 	gtk_text_buffer_move_mark_by_name (buffer, "selection_bound", &iter_match_start);
 	gtk_text_buffer_move_mark_by_name (buffer, "insert", &iter_match_end);
+
+	return TRUE;
+}
+
+
+void
+gossip_chat_view_find_abilities (GossipChatView *view,
+				 const gchar    *search_criteria,
+				 gboolean       *can_do_previous,
+				 gboolean       *can_do_next)
+{
+	GossipChatViewPriv *priv;
+	GtkTextBuffer      *buffer;
+	GtkTextIter         iter_at_mark;
+	GtkTextIter         iter_match_start;
+	GtkTextIter         iter_match_end;
+
+	g_return_if_fail (GOSSIP_IS_CHAT_VIEW (view));
+	g_return_if_fail (search_criteria != NULL);
+	g_return_if_fail (can_do_previous != NULL && can_do_next != NULL);
+
+	priv = GET_PRIV (view);
+
+	buffer = priv->buffer;
+
+	if (can_do_previous) {
+		if (priv->find_mark_previous) {
+			gtk_text_buffer_get_iter_at_mark (buffer,
+							  &iter_at_mark,
+							  priv->find_mark_previous);
+		} else {
+			gtk_text_buffer_get_start_iter (buffer, &iter_at_mark);
+		}
+		
+		*can_do_previous = gossip_text_iter_backward_search (&iter_at_mark,
+								     search_criteria,
+								     &iter_match_start,
+								     &iter_match_end,
+								     NULL);
+	}
+
+	if (can_do_next) {
+		if (priv->find_mark_next) {
+			gtk_text_buffer_get_iter_at_mark (buffer,
+							  &iter_at_mark,
+							  priv->find_mark_next);
+		} else {
+			gtk_text_buffer_get_start_iter (buffer, &iter_at_mark);
+		}
+		
+		*can_do_next = gossip_text_iter_forward_search (&iter_at_mark,
+								search_criteria,
+								&iter_match_start,
+								&iter_match_end,
+								NULL);
+	}
 }
 
 void
@@ -1962,7 +2254,7 @@ gossip_chat_view_highlight (GossipChatView *view,
 					    &iter_start,
 					    &iter_end);
 
-	if (!text || strlen (text) < 1) {
+	if (G_STR_EMPTY (text)) {
 		return;
 	}
 
@@ -2086,6 +2378,8 @@ gossip_chat_view_get_smiley_menu (GCallback    callback,
 	gint       y;
 	gint       i;
 
+	g_return_val_if_fail (callback != NULL, NULL);
+
 	menu = gtk_menu_new ();
 
 	for (i = 0, x = 0, y = 0; i < GOSSIP_SMILEY_COUNT; i++) {
