@@ -16,10 +16,6 @@
  * License along with this program; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
- * 
- * Authors: Mikael Hallendal <micke@imendio.com>
- *          Richard Hult <richard@imendio.com>
- *          Martyn Russell <martyn@imendio.com>
  */
 
 #include "config.h"
@@ -42,6 +38,7 @@
 #include "gossip-app.h"
 #include "gossip-chatrooms-window.h"
 #include "gossip-edit-chatroom-dialog.h"
+#include "gossip-glade.h"
 #include "gossip-new-chatroom-dialog.h"
 #include "gossip-ui-utils.h"
 
